//* Copyright (c) 2009, toutenvrac.org - ouiouioui
//* All rights reserved.
//* Redistribution and use in source and binary forms, with or without
//* modification, are permitted provided that the following conditions are met:
//*
//*     * Redistributions of source code must retain the above copyright
//*       notice, this list of conditions and the following disclaimer.
//*     * Redistributions in binary form must reproduce the above copyright
//*       notice, this list of conditions and the following disclaimer in the
//*       documentation and/or other materials provided with the distribution.
//*     * Neither the name toutenvrac.org, ouiouioui the names of its
//*       contributors may be used to endorse or promote products derived from
//*       this software without specific prior written permission but if you
//*       email at contact@toutenvrac.org for say what you make with it would
//*       be nice.
//*
//* THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
//* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//* DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
//* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
//* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
Unit TCheckDD;

Interface

Uses
  Classes, Math, Dialogs, SysUtils,
{$IFDEF MSWINDOWS}
  Windows
{$ENDIF}
  ;

Type
  TTCheckDD = Class(TThread)
  Private
    Space:          Integer;
    Path:           PAnsiChar;
    Msg, StatusMsg: String;
    Procedure SetName;
    Procedure SetStopAll(AMsg, AStatusMsg: String);
    Procedure SynchroStopAll;
  Protected
    Procedure Execute; Override;
  Public
    Constructor Create(ASpace: Integer; APath: String);
  End;

Implementation

Uses
  MainForm;

{ Important : les mthodes et proprits des objets de la VCL peuvent uniquement tre
  utiliss dans une mthode appele en utilisant Synchronize, comme : 

      Synchronize(UpdateCaption);

  o UpdateCaption serait de la forme

    procedure TTCheckDD.UpdateCaption;
    begin
      Form1.Caption := 'Mis  jour dans un thread';
    end; }

{$IFDEF MSWINDOWS}
Type
  TThreadNameInfo = Record
    FType:  Longword;     // doit tre 0x1000
    FName:  PChar;        // pointeur sur le nom (dans l'espace d'adresse de l'utilisateur)
    FThreadID: Longword; // ID de thread (-1=thread de l'appelant)
    FFlags: Longword;    // rserv pour une future utilisation, doit tre zro
  End;

{$ENDIF}

{ TTCheckDD }

Procedure TTCheckDD.SetName;
{$IFDEF MSWINDOWS}
Var
  ThreadNameInfo: TThreadNameInfo;
{$ENDIF}
Begin
{$IFDEF MSWINDOWS}
  ThreadNameInfo.FType  := $1000;
  ThreadNameInfo.FName  := 'TCheckDD';
  ThreadNameInfo.FThreadID := $FFFFFFFF;
  ThreadNameInfo.FFlags := 0;

  Try
    RaiseException($406D1388, 0, sizeof(ThreadNameInfo) Div sizeof(Longword), @ThreadNameInfo);
  Except
  End;
{$ENDIF}
End;

Constructor TTCheckDD.Create(ASpace: Integer; APath: String);
Begin
  Inherited Create(False);
  FreeOnTerminate := True;
  Space := ASpace;
  Path  := PChar(APath);
End;

Procedure TTCheckDD.Execute;
Var
  Nb: Integer;
  FreeAvailable, TotalSpace: TLargeInteger;
Begin
  SetName;
  While Not Terminated Do
  Begin
    //- sleep 1sec
    Nb := 0;
    While (Not Terminated) And (Nb < 10) Do
    Begin
      Sleep(100);
      Inc(Nb);
    End;
    //- check space restant min is space
    GetDiskFreeSpaceEx(Path, FreeAvailable, TotalSpace, nil);
    FreeAvailable := FreeAvailable Div 1024 Div 1024 Div 1024;
    TotalSpace := TotalSpace Div 1024 Div 1024 Div 1024;
    If FreeAvailable < Space Then
      SetStopAll('Espace disque insuffisant il reste ' + IntToStr(FreeAvailable) + 'Go sur ' + IntToStr(TotalSpace) + 'Go' + sLineBreak +
        'Vous devez librer de la place si vous voulez' + sLineBreak +
        'enregistrer de nouvelles images et/ou vidos.', 'Espace disque insuffisant il reste ' + IntToStr(FreeAvailable) + 'Go sur ' + IntToStr(TotalSpace) + 'Go');
  End;
End;

Procedure TTCheckDD.SetStopAll(AMsg, AStatusMsg: String);
Begin
  Msg := AMsg;
  StatusMsg := AStatusMsg;
  Synchronize(SynchroStopAll);
End;

Procedure TTCheckDD.SynchroStopAll;
Begin
  Main.StopAll(Msg, StatusMsg);
End;

End.

