//******************************************************************************
//  TVideoCapture -  wrapper for DirectShow video capture functions
//  Based on
//  Microsoft's AMCap & StillCap
//  and
//  DScapture by orthkon * www.mp3.com/orthkon * orthkon@mail.com
//
//  written by Egor Averchenkov, 2001-2002,
//             e_g_o_r@mail.ru
//
//  version 1.15
//******************************************************************************
//  License Agreement
//
//  Permission to use, copy, modify, and distribute this software and its
//  documentation for any purpose and without fee is hereby granted,
//  provided that the above copyright notice appears in all copies and
//  that both the above copyright notice and this permission notice appear
//  in supporting documentation, and that the name the author
//  not be used in advertising or publicity pertaining to distribution of
//  the software without specific, written prior permission. This
//  software is made available "as is", and AUTHOR DISCLAIM
//  ALL WARRANTIES, EXPRESS OR IMPLIED, WITH REGARD TO THIS SOFTWARE,
//  INCLUDING WITHOUT LIMITATION ALL IMPLIED WARRANTIES OF MERCHANTABILITY
//  AND FITNESS FOR A PARTICULAR PURPOSE, AND IN NO EVENT SHALL AUTHORS BE
//  LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
//  DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
//  WHETHER IN AN ACTION OF CONTRACT, TORT (INCLUDING NEGLIGENCE) OR
//  STRICT LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//  PERFORMANCE OF THIS SOFTWARE.
//
//  In other words - feel free to use this code)))
//  I hope someone find it usefull. If you do - send me a message, pls.
//  (Beer messages would be appreciated a lot!!! :o) )
//******************************************************************************
//  History log:

//  01-04-24 - v1.00 (initial release)

//  01-09-18 - v1.02 -  I had to continue this work after a long delay
//    some stupid bugs corrected
//    interface section was slightly ordered
//    new property CapturePixelFormat added to select color depth of captured bitmaps

//  01-09-20 - v1.03 - orthkon mailed me TCapture based on this component - good work!
//    added properties and functions to work with available capture modes

//  01-09-22 - v1.04
//    working with filters property pages

//  01-10-18 - v1.05
//    working with compressors        if ParCam[1].Vignette.Visible then  if ParCam[1].Vignette.Visible then

//  01-10-27 - v1.06 - thanks to Lee_Nover (Lee.Nover@email.si) - he did all dirty work))))
//    one graph for preview/capture
//    compression dialogs

//  01-10-28 - v1.07
//    bugs in building graph with compressors were corrected

//  01-11-23 - v1.08
//    completely rewriten graph building/controlling functions
//    new properties:
//        IsDV - Indicates DV Capture source is used
//        WantDVAudio - Force using DV Audio stream to capture audio
//                      when DV device is used for capturing

//    Sorry but I've changed some names(((

//  01-11-25 - v1.09
//    saving/restoring capture graph configuration

//  02-05-25 - v1.10
//    some bugs corrected
//    graph restoring/building algorithms slightly ordered
//    new properties:
//      VCompDevice - name of video compressor used
//      ACompDevice - name of audio compressor used
//      VCapDevice  - name of video capture device used
//      ACapDevice  - name of audio capture device used
//      VCapIndex   - index of video capture device used
//      ACapIndex   - index of audio capture device used
//      VCompIndex  - index of video compressor device used
//      ACompIndex  - index of audio compressor device used
//      ACapModeIdx - index of audio capture mode used
//      VCapModeIdx - index of video capture mode used

//  02-10-10 - v1.11
//    1. added SampleGrabber buffer locking functions to avoid
//       simultaneous access from different threads to buffer data
//    2. added OnFramePassed event firing every time frame comes
//       through SampleGrabber

//  02-10-13 - v1.12
//    only BCB support added to sources

//  02-10-16 - v1.13
//    added WantAudioPreview property to disable audio preview rendering
//    added OnAborted event to indicate graph run aborted for some reason
//    added OnDeviceLost event to indicate capture device lost during preview/capture
//    some optimization done

//  02-10-20 - v1.14
//    some bugs in working with the capture and compressor lists corrected

//  02-11-03 - v1.15
//    bugs when working with DV capture devices corrected
//    bug in rendering graph with WantAudioPreview property corrected
//    bug in working with the capture and compressor lists corrected
//******************************************************************************
//  Known problems.

//  Some audio compressor filters do not expose IAMStreamControl
//  (it's pitty but ACM Wrapper does not((( ) so component can't correctly
//  start/stop capture part of graph using such filters.
//  It's recomended to use native DShow audio compressors to capture avi.
//******************************************************************************
Unit VCap;

Interface

// -----------------------------------------------------------------------------
// BCB support
{$NOINCLUDE ActiveX}
{$NOINCLUDE DirectShow}
{$HPPEMIT '/* don''t use DirectShow.pas in C++ sources - use native C++ headers!!! */' }
{$HPPEMIT '#define Directshow ' }
{$HPPEMIT '#include <strmif.h>' }
{$HPPEMIT '#include <control.h>' }
{$HPPEMIT 'typedef System::DelphiInterface<IGraphBuilder> _di_IGraphBuilder;'}
{$HPPEMIT 'typedef System::DelphiInterface<ICaptureGraphBuilder2> _di_ICaptureGraphBuilder2;' }
{$HPPEMIT 'typedef System::DelphiInterface<IVideoWindow> _di_IVideoWindow;' }
{$HPPEMIT 'typedef System::DelphiInterface<IMediaEventEx> _di_IMediaEventEx;' }
{$HPPEMIT 'typedef System::DelphiInterface<IAMDroppedFrames> _di_IAMDroppedFrames;' }
{$HPPEMIT 'typedef System::DelphiInterface<IAMVfwCaptureDialogs> _di_IAMVfwCaptureDialogs;' }
{$HPPEMIT 'typedef System::DelphiInterface<IAMVfwCompressDialogs> _di_IAMVfwCompressDialogs;' }
{$HPPEMIT 'typedef System::DelphiInterface<IAMStreamConfig> _di_IAMStreamConfig;' }
{$HPPEMIT 'typedef System::DelphiInterface<IBaseFilter> _di_IBaseFilter;' }
{$HPPEMIT 'typedef System::DelphiInterface<IFileSinkFilter> _di_IFileSinkFilter;' }
{$HPPEMIT 'typedef System::DelphiInterface<IConfigAviMux> _di_IConfigAviMux;' }
{$HPPEMIT 'typedef System::DelphiInterface<IUnknown> _di_ISampleGrabber; // to use ISampleGrabber include <qedit.h>' }

Uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, ExtCtrls,
  ActiveX, DirectShow;

// -----------------------------------------------------------------------------
// Windows messages
Const
  WM_FGNOTIFY       = WM_USER + 1;
  WM_SAMPLECAPTURED = WM_USER + 2;
  WM_SAMPLEPASSED   = WM_USER + 3;

// message WM_SAMPLECAPTURED
Type
  TWMSampleCaptured = Packed Record
    Msg:       Cardinal;
    GrabberCB: TObject;
    LParam:    Longint;
    Result:    Longint;
  End;

// -----------------------------------------------------------------------------
Type
  EVideoCaptureError = Class(Exception);

Type
  TCapturedBitmap = Class(TBitmap); // to eliminate some problems with BCB

Type
  TBitmapCapturedEvent = Procedure(CapturedImage: TCapturedBitmap) Of Object;

Type
  TDVResolution = (dvrDontWorry, dvrFull, dvrHalf, dvrQuater, dvrDC);

// -----------------------------------------------------------------------------
// Working with capture modes
Type
  TVCapMode = Record
    MediaType:    TGUID;
    MediaSubType: TGUID;
    Width:        Integer;
    Height:       Integer;
    BitCount:     Integer;
    FrameRate,
    MinFrameRate,
    MaxFrameRate: Double;
  End;

Type
  TACapMode = Record
    MediaType:     TGUID;
    MediaSubType:  TGUID;
    SampleRate:    Integer;
    SampleSize:    Integer;
    Channels:      Integer;
    AvgSampleRate: Integer;
  End;

Function IsEqualModes(Const mode1, mode2: TVCapMode): Boolean; Overload;
Function IsEqualModes(Const mode1, mode2: TACapMode): Boolean; Overload;

Function GetModeString(Const mode: TVCapMode): String; Overload;
Function GetModeString(Const mode: TACapMode): String; Overload;

// -----------------------------------------------------------------------------
// Various Capture Dialog Types
Type
  TCaptureDialog  = (
    cdVFORMAT,
    cdVSOURCE,
    cdVDISPLAY,
    cdVCAPTURE,
    cdVCROSSBAR,
    cdTVTUNER,
    cdACAPTURE,
    cdACROSSBAR,
    cdTVAUDIO,
    cdVCAPTURE_PIN,
    cdVPREVIEW_PIN,
    cdACAPTURE_PIN,
    cdVCOMPRESSION,
    cdACOMPRESSION
    );
  TCaptureDialogs = Set Of TCaptureDialog;

// -----------------------------------------------------------------------------
// TVideoCapture component
Type

  TGraphConfig = Class;

  TVideoCapture = Class(TCustomControl)
  Private
    Graph:           IGraphBuilder;
    Builder:         ICaptureGraphBuilder2;
    VideoWindow:     IVideoWindow;
    MediaEvent:      IMediaEventEx;
    DroppedFrames:   IAMDroppedFrames;
    CaptureDialogs:  IAMVfwCaptureDialogs;
    CompressDialogs: IAMVfwCompressDialogs;
    AStreamConf:     IAMStreamConfig;   // for audio capture
    VStreamConf:     IAMStreamConfig;   // for video capture

    Render:            IBaseFilter;            // file writer
    VCap, ACap,                                // capture filters
    VComp, AComp,                              // compressor filters
    DVSplitter, DVDec: IBaseFilter;            // filters to use with DV devices
    Sink:              IFileSinkFilter;
    NullRenderer:      IBaseFilter;
    ConfigAviMux:      IConfigAviMux;
    Grabber:           ISampleGrabber;

    GrabberCB: TObject;

    FCCAvail:    Boolean;
    FCapCC:      Boolean;
    FGraphBuilt: Boolean;

    FACapFriendlyName, FVCapFriendlyName: String;

    FCapturing:    Boolean;
    FPreviewing:   Boolean;
    FUseFrameRate: Boolean;
    FUseTimeLimit: Boolean;
    FFrameRate:    Double;
    FCapStartTime: DWORD;
    FCapStopTime:  DWORD;
    FMasterStream: Integer;

    FNotDropped:     Integer;
    FDroppedFrames:  Integer;
    FNotDroppedBase: Integer;
    FDroppedBase:    Integer;
    FCapTime:        DWORD;

    mVideo, mAudio, mVComp, mAComp: IMoniker;
    FCaptureTimer:                  TTimer;

    FDialogs:        TCaptureDialogs;
    FDoPreallocFile: Boolean;

    FTempCaptureFileName: String;
    FCaptureFileName:     String;
    FTimeLimit:           Integer;
    FUseTempFile:         Boolean;
    FPreallocFileSize:    Cardinal;
    FCaptureFile:         WideString;
    FDVResolution:        TDVResolution;

    FOnStopPreview:      TNotifyEvent;
    FOnStartPreview:     TNotifyEvent;
    FOnStopCapture:      TNotifyEvent;
    FOnStartCapture:     TNotifyEvent;
    FOnChangeDevice:     TNotifyEvent;
    FOnCaptureProgress:  TNotifyEvent;
    FOnBitmapGrabbed:    TBitmapCapturedEvent;
    FOnChangeACapMode:   TNotifyEvent;
    FOnChangeVCapMode:   TNotifyEvent;
    FOnChangeCompressor: TNotifyEvent;
    FOnFrameRendered:    TNotifyEvent;
    FOnDeviceLost:       TNotifyEvent;
    FOnAborted:          TNotifyEvent;

    FPixelFormat:      TPixelFormat;
    FWantPreview:      Boolean;
    FWantCapture:      Boolean;
    FWantAudio:        Boolean;
    FWantDVAudio:      Boolean;
    FWantBitmaps:      Boolean;
    FWantAudioPreview: Boolean;

    Procedure SetMasterStream(Const Value: Integer);
    Property MasterStream: Integer Read FMasterStream Write SetMasterStream;
    Procedure SetFrameRate(Const Value: Double);
    Procedure SetTempCaptureFileName(Const Value: String);
    Procedure SetCaptureFileName(Const Value: String);
    Function GetACapMode: TACapMode;
    Function GetACapModeCount: Integer;
    Function GetACapModes(i: Integer): TACapMode;
    Function GetVCapMode: TVCapMode;
    Function GetVCapModeCount: Integer;
    Function GetVCapModes(i: Integer): TVCapMode;
    Function GetIsDVSource: Boolean;
    Procedure SetDVResolution(Const Value: TDVResolution);
    Function GetACapSource: String;
    Function GetVCapSource: String;
    Function GetACompDevice: String;
    Function GetVCompDevice: String;
    Function GetACapSourceIdx: Integer;
    Function GetACompDeviceIdx: Integer;
    Function GetVCapSourceIdx: Integer;
    Function GetVCompDeviceIdx: Integer;
    Function GetACapModeIdx: Integer;
    Function GetVCapModeIdx: Integer;
  Private
    Procedure SetSize(Var msg: TMessage); Message WM_SIZE;  // Changing size of cap window
    Procedure GraphEvent(Var msg: TMessage); Message WM_FGNOTIFY;
    Procedure BitmapGrabbed(Var msg: TWMSampleCaptured); Message WM_SAMPLECAPTURED;
    Procedure FramePassed(Var msg: TMessage); Message WM_SAMPLEPASSED; // sample passed through SampleGrabber
    Procedure CaptureProgress(Sender: TObject);
    Procedure ResizeWindow;
    Procedure UpdateStatus;
    Procedure ChooseDevices(nmVideo, nmAudio: IMoniker; ForceGraphRebuild: Boolean = False); Overload;
    Procedure ChooseCompressors(nmVComp, nmAComp: IMoniker; ForceGraphRebuild: Boolean = False); Overload;
    Procedure DoAborted(hr: HRESULT);
    Procedure DoDeviceLost;

    Procedure CleanUp;
    Procedure FreeCapFilters;
    Function InitCapFilters: Boolean;
    Function MakeBuilder: Boolean;
    Function MakeGraph: Boolean;
    Procedure NukeDownstream(pf: IBaseFilter);
    Procedure TearDownGraph;
    Function FindVideoWindow: Boolean;
    Function RenderVideoPreview(Const VSource: IBaseFilter): Boolean;
    Function BuildGraph: Boolean;
    Function ControlCaptureStream(start: Boolean): Boolean;

    Function AllocCaptureFile(Const SizeMb: Integer): Boolean;
//    Function SaveCaptureFile(Const FileName: String): Boolean;
    Procedure GetAvailableDialogs;
  Public
    // if someone wants to work with DShow objects
    Property CaptureGraph: IGraphBuilder Read Graph;
    Property VCapFilter: IBaseFilter Read VCap;
    Property ACapFilter: IBaseFilter Read ACap;
    Property VCompFilter: IBaseFilter Read VComp;
    Property ACompFilter: IBaseFilter Read AComp;

                                                   // graph state properties
    Property Capturing: Boolean Read FCapturing;   // is capturing now
    Property Previewing: Boolean Read FPreviewing; // is previewing now

    // video capture properties
    Property VCapFriendlyName: String Read FVCapFriendlyName;
    Property VCapName: String Read GetVCapSource; // used video capture device name
    Property VCapIndex: Integer Read GetVCapSourceIdx;

    Property VCapMode: TVCapMode Read GetVCapMode; // current video capture mode
    Property VCapModeIdx: Integer Read GetVCapModeIdx; // current video capture mode index
    Property VCapModeCount: Integer Read GetVCapModeCount; // available video capture mode count
    Property VCapModes[i: Integer]: TVCapMode Read GetVCapModes;
    Function SetVCapMode(i: Integer): Boolean; Overload;
    Function SetVCapMode(Const mode: TVCapMode): Boolean; Overload;

    // audio capture properties
    Property ACapFriendlyName: String Read FACapFriendlyName;
    Property ACapName: String Read GetACapSource; // used audio capture device name
    Property ACapIndex: Integer Read GetACapSourceIdx;

    Property ACapMode: TACapMode Read GetACapMode; // current audio capture mode
    Property ACapModeIdx: Integer Read GetACapModeIdx; // current audio capture mode index
    Property ACapModeCount: Integer Read GetACapModeCount; // available audio capture mode count
    Property ACapModes[i: Integer]: TACapMode Read GetACapModes;
    Function SetACapMode(i: Integer): Boolean; Overload;
    Function SetACapMode(Const mode: TACapMode): Boolean; Overload;

                                                         // compressors properties
    Property VCompName: String Read GetVCompDevice;      // used video compressor name
    Property VCompIndex: Integer Read GetVCompDeviceIdx; // used video compressor index
    Property ACompName: String Read GetACompDevice;      // used audio compressor name
    Property ACompIndex: Integer Read GetACompDeviceIdx; // used audio compressor index

    // other properties
    Property IsDVSource: Boolean Read GetIsDVSource; // indicates DV capture source is used
    Property Dialogs: TCaptureDialogs Read FDialogs; // available graph dialogs
    Property FramesDropped: Integer Read FDroppedFrames;
    Property NotDropped: Integer Read FNotDropped;
    Property CapStartTime: DWORD Read FCapStartTime;
    Property CapStopTime: DWORD Read FCapStopTime;
    Property CapTime: DWORD Read FCapTime;

    Procedure ChooseDevices(Const szVideo, szAudio: String; ForceGraphRebuild: Boolean = False); Overload;
    Procedure ChooseDevices(Const numVideo, numAudio: Integer; ForceGraphRebuild: Boolean = False); Overload;

    Procedure ChooseCompressors(Const szVComp, szAComp: String; ForceGraphRebuild: Boolean = False); Overload;
    Procedure ChooseCompressors(Const numVComp, numAComp: Integer; ForceGraphRebuild: Boolean = False); Overload;

    Function Init: Boolean; // you have to call this routine first!!!
    Function StartPreview: Boolean;
    Function StopPreview: Boolean;
    Function StartCapture(Const Dialog: Boolean = False): Boolean;
    Function StopCapture(RenameFile: String = ''): Boolean;
    Function CaptureFrame: Boolean;

    Function ShowDialog(Const DialogType: TCaptureDialog): Boolean;

    Procedure SaveGraph(config: TGraphConfig);
    Procedure RestoreGraph(config: TGraphConfig);

    Constructor Create(AOwner: TComponent); Override;
    Destructor Destroy; Override;
  Published
    Property DVResolution: TDVResolution Read FDVResolution Write SetDVResolution;

    // set to true if you want to capture single frames during preview
    Property WantBitmaps: Boolean Read FWantBitmaps Write FWantBitmaps;
    Property BitmapPixelFormat: TPixelFormat Read FPixelFormat Write FPixelFormat;

    Property WantAudio: Boolean Read FWantAudio Write FWantAudio; // set true if you want audio in captured file
    Property WantDVAudio: Boolean Read FWantDVAudio Write FWantDVAudio; // set true if you want DV audio in captured file
    Property WantAudioPreview: Boolean Read FWantAudioPreview Write FWantAudioPreview; // set true if you want render audio preview
    Property WantPreview: Boolean Read FWantPreview Write FWantPreview;
    Property WantCapture: Boolean Read FWantCapture Write FWantCapture; // set to false if dont want capture file is being created
    Property UseFrameRate: Boolean Read FUseFrameRate Write FUseFrameRate;
    Property FrameRate: Double Read FFrameRate Write SetFrameRate;
    Property CaptureFileName: String Read FCaptureFileName Write SetCaptureFileName;
    Property UseTempFile: Boolean Read FUseTempFile Write FUseTempFile;
    Property TempCaptureFileName: String Read FTempCaptureFileName Write SetTempCaptureFileName;
    Property UseTimeLimit: Boolean Read FUseTimeLimit Write FUseTimeLimit;
    Property TimeLimit: Integer Read FTimeLimit Write FTimeLimit;
    Property DoPreallocFile: Boolean Read FDoPreallocFile Write FDoPreallocFile;
    Property PreallocFileSize: Cardinal Read FPreallocFileSize Write FPreallocFileSize Default 10;

    Property OnAborted: TNotifyEvent Read FOnAborted Write FOnAborted; // if normal graph run has been aborted
    Property OnDeviceLost: TNotifyEvent Read FOnDeviceLost Write FOnDeviceLost;
    Property OnChangeDevice: TNotifyEvent Read FOnChangeDevice Write FOnChangeDevice;
    Property OnChangeCompressor: TNotifyEvent Read FOnChangeCompressor Write FOnChangeCompressor;
    Property OnChangeVCapMode: TNotifyEvent Read FOnChangeVCapMode Write FOnChangeVCapMode;
    Property OnChangeACapMode: TNotifyEvent Read FOnChangeACapMode Write FOnChangeACapMode;
    Property OnCaptureProgress: TNotifyEvent Read FOnCaptureProgress Write FOnCaptureProgress;
    Property OnStopCapture: TNotifyEvent Read FOnStopCapture Write FOnStopCapture;
    Property OnStopPreview: TNotifyEvent Read FOnStopPreview Write FOnStopPreview;
    Property OnStartCapture: TNotifyEvent Read FOnStartCapture Write FOnStartCapture;
    Property OnStartPreview: TNotifyEvent Read FOnStartPreview Write FOnStartPreview;
    Property OnBitmapGrabbed: TBitmapCapturedEvent Read FOnBitmapGrabbed Write FOnBitmapGrabbed;
    Property OnFrameRendered: TNotifyEvent Read FOnFrameRendered Write FOnFrameRendered;
  Published
    Property Align;
    Property Color;
    Property Visible;
    Property OnMouseMove;
    Property OnMouseUp;
    Property OnMouseDown;
    Property OnClick;
    Property OnDblClick;
  End;

  TGraphConfig = Class
  Public
    VCapSource: String;
    ACapSource: String;
    VComp:      String;
    AComp:      String;
    ACapMode:   TACapMode;
    VCapMode:   TVCapMode;
    VCompState: String;

    WantCapture:      Boolean;
    WantPreview:      Boolean;
    WantBitmaps:      Boolean;
    WantAudio:        Boolean;
    WantDVAudio:      Boolean;
    WantAudioPreview: Boolean;

    PixelFormat:  TPixelFormat;
    DVResolution: TDVResolution;

    CaptureFileName:     String;
    UseTempFile:         Boolean;
    TempCaptureFileName: String;
    DoPreallocFile:      Boolean;
    PreallocFileSize:    Cardinal;

    Procedure Clear;
    Procedure RestoreGraph(layout: String);
    Function RestoreGraphFromStream(Stream: TStream): Boolean;
    Function SaveGraph: String;
    Function SaveGraphToStream(Stream: TStream): Boolean;

    Constructor Create;
  End;

// device enum functions
// callers have to free aquired lists!!!
Function GetVideoDevicesList(Const Refresh: Boolean = False): TStringList;
Function GetAudioDevicesList(Const Refresh: Boolean = False): TStringList;
Function GetVideoCompressorsList(Const Refresh: Boolean = False): TStringList;
Function GetAudioCompressorsList(Const Refresh: Boolean = False): TStringList;

Procedure Register;

Implementation

Uses
  VCapStrings, Utils, Dialogs, MMSystem, contnrs, syncobjs;

//-----------------------------------------------------------------------
Procedure Register;
Begin
  RegisterComponents('Samples', [TVideoCapture]);
End;

//-----------------------------------------------------------------
//const
//  MAX_TIME: TReference_Time = $7FFFFFFFFFFFFFFF;

// my filter names to find these filters in the graph
Const
  VCompFilterName  = 'MyVideoCompressor';
  ACompFilterName  = 'MyAudioCompressor';
  DVSplitterFilterName = 'MyDVSplitter';
  DVDecoderFilterName = 'MyDVDecoder';
  SmartTeeFilterName = 'MySmartTee';
  InfTeeFilterName = 'MyInfTee';
  SampleGrabberFilterName = 'MyBitmapGrabber';
  NullRendererFilterName = 'MyNullRenderer';

Const
  IID_IPropertyBag: TGUID = '{55272A00-42CB-11CE-8135-00AA004BB851}';
  IID_ISpecifyPropertyPages: TGUID = '{B196B28B-BAB4-101A-B69C-00AA00341D07}';

Const
  VfwCompressDialog_Config = 1;
  VfwCompressDialog_About  = 2;
  // returns S_OK if the dialog exists and can be shown, else S_FALSE
  VfwCompressDialog_QueryConfig = 4;
  VfwCompressDialog_QueryAbout = 8;

Const
  AM_FILE_OVERWRITE = 1;

// DV Modes
Const
  DVResolutions: Array [TDVResolution] Of Integer = (
    0,
    DVRESOLUTION_FULL,
    DVRESOLUTION_HALF,
    DVRESOLUTION_QUARTER,
    DVRESOLUTION_DC
    );

Type
  TDVRes = Record
    DVWidth,
    DVHeight,
    DVBits: Integer;
  End;

Const
  DVRes: Array [DVRESOLUTION_FULL..DVRESOLUTION_DC] Of TDVRes = (
    (DVWidth: 720; DVHeight: 576; DVBits: 24), // Full
    (DVWidth: 360; DVHeight: 288; DVBits: 24), // Half
    (DVWidth: 180; DVHeight: 144; DVBits: 24), // quater
    (DVWidth: 88; DVHeight: 72; DVBits: 24)    // DC
    );

Const
  DVRes1: Array [DVRESOLUTION_FULL..DVRESOLUTION_DC] Of TDVResolution = (
    dvrFull,
    dvrHalf,
    dvrQuater,
    dvrDC
    );

Const
  DVCapModesCount = High(DVRes) - Low(DVRes) + 1;

// used to install SampleGrabber
Const
  PixelFormatGUIDs: Array [TPixelFormat] Of PGUID = (@MEDIASUBTYPE_RGB24, @MEDIASUBTYPE_RGB1, @MEDIASUBTYPE_RGB4, @MEDIASUBTYPE_RGB8, @MEDIASUBTYPE_RGB555, @MEDIASUBTYPE_RGB565, @MEDIASUBTYPE_RGB24, @MEDIASUBTYPE_RGB32, @MEDIASUBTYPE_RGB24
    );

//-----------------------------------------------------------------
Function MyMsg(szMsg: String; hr: HRESULT): String;
Begin
  Result := Format(szMsg, [hr]);
  MessageBox(GetForegroundWindow, PChar(Result), PChar(rsDShowCapture), MB_OK Or MB_ICONSTOP);
End;

Procedure ErrMsg(szMsg: String; hr: HRESULT = 0);
Begin
  MyMsg(szMsg, hr);
End;

Procedure ErrMsgException(szMsg: String; hr: HRESULT = 0);
Begin
  Raise EVideoCaptureError.Create(MyMsg(szMsg, hr));
End;

//-----------------------------------------------------------------
Function CheckGUID(Const p1, p2: TGUID): Boolean;
Var
  i: Byte;
Begin
  Result := False;
  For i := 0 To 7 Do
    If p1.D4[i] <> p2.D4[i] Then
      Exit;
  Result := (p1.D1 = p2.D1) And (p1.D2 = p2.D2) And (p1.D3 = p2.D3);
End;

// Free an existing media type (ie free resources it holds)
Procedure FreeMediaType(mt: TAM_MEDIA_TYPE);
Begin
  If mt.cbFormat <> 0 Then
  Begin
    CoTaskMemFree(mt.pbFormat);
    // Strictly unnecessary but tidier
    mt.cbFormat := 0;
    mt.pbFormat := nil;
  End;
  mt.pUnk := nil;
End;

Procedure DeleteMediaType(pmt: PAM_MEDIA_TYPE);
Begin
  // allow NULL pointers for coding simplicity
  If pmt <> nil Then
  Begin
    FreeMediaType(pmt^);
    CoTaskMemFree(pmt);
  End;
End;

//-----------------------------------------------------------------
Function IsEqualModes(Const mode1, mode2: TVCapMode): Boolean;
Begin
  Result := CheckGUID(mode1.MediaSubType, mode2.MediaSubType) And
    (mode1.Height = mode2.Height) And
    (mode1.Width = mode2.Width) And
    (mode1.BitCount = mode2.BitCount);
End;

Function IsEqualModes(Const mode1, mode2: TACapMode): Boolean;
Begin
  Result := CheckGUID(mode1.MediaSubType, mode2.MediaSubType);
  If Result Then
  Begin
    If CheckGUID(mode1.MediaSubType, MEDIASUBTYPE_PCM) Then
      Result := (mode1.SampleRate = mode2.SampleRate) And
        (mode1.SampleSize = mode2.SampleSize) And
        (mode1.Channels = mode2.Channels)
    Else
      Result := (mode1.AvgSampleRate = mode2.AvgSampleRate) And
        (mode1.SampleRate = mode2.SampleRate) And
        (mode1.Channels = mode2.Channels);
  End;
End;

Function GetModeString(Const mode: TVCapMode): String; Overload;
Type
  TSubTypeName = Record
    SubType: PGUID;
    Name: String;
  End;
Const
  MediaSubTypeNames: Array [1..13] Of TSubTypeName =
    ((SubType: @MEDIASUBTYPE_YVU9; Name: '(YVU9)'),
    (SubType: @MEDIASUBTYPE_Y411; Name: '(Y411)'),
    (SubType: @MEDIASUBTYPE_Y41P; Name: '(Y41P)'),
    (SubType: @MEDIASUBTYPE_YUY2; Name: '(YUY2)'),
    (SubType: @MEDIASUBTYPE_YVYU; Name: '(YVYU)'),
    (SubType: @MEDIASUBTYPE_UYVY; Name: '(UYVY)'),
    (SubType: @MEDIASUBTYPE_Y211; Name: '(Y211)'),
    (SubType: @MEDIASUBTYPE_RGB24; Name: '(RGB24)'),
    (SubType: @MEDIASUBTYPE_RGB32; Name: '(RGB32)'),
    (SubType: @MEDIASUBTYPE_RGB565; Name: '(RGB565)'),
    (SubType: @MEDIASUBTYPE_RGB555; Name: '(RGB555)'),
    (SubType: @MEDIASUBTYPE_ARGB32; Name: '(ARGB32)'),
    (SubType: @MEDIASUBTYPE_RGB565; Name: '(RGB565)'));
Var
  i: Integer;
Begin
  Result := Format('%dx%dx%db', [mode.Width, mode.Height, mode.BitCount]);
  For i := low(MediaSubTypeNames) To high(MediaSubTypeNames) Do
    If CheckGUID(mode.MediaSubType, MediaSubTypeNames[i].SubType^) Then
    Begin
      Result := Result + ' ' + MediaSubTypeNames[i].Name;
      break;
    End;
End;

Function GetModeString(Const mode: TACapMode): String; Overload;
Begin
  If CheckGUID(mode.MediaSubType, MEDIASUBTYPE_PCM) Then
  Begin
    Result := Format('%dHzx%dbx%d', [mode.SampleRate, mode.SampleSize, mode.Channels]);
  End
  Else
  Begin
    Result := Format('%dkBits/s, %dHz, %d', [mode.AvgSampleRate * 8 Div 1000, mode.SampleRate, mode.Channels]);
  End;
End;

//-----------------------------------------------------------------
{ TCapDeviceInfo }
Type
  TCapDeviceInfo = Class
  Public
    DeviceName: String;
    DeviceNameInList: String;
    Moniker: IMoniker;
    Constructor Create(Const aDeviceName, aDeviceNameInList: String; Const aMoniker: IMoniker);
    Destructor Destroy; Override;
  End;

Constructor TCapDeviceInfo.Create(Const aDeviceName, aDeviceNameInList: String; Const aMoniker: IMoniker);
Begin
  DeviceName := aDeviceName;
  DeviceNameInList := aDeviceNameInList;
  Moniker := aMoniker;
End;

Destructor TCapDeviceInfo.Destroy;
Begin
  Moniker := nil; // I guess we have to release moniker - tnx to Gabriel Corneanu
  Inherited;
End;

//-----------------------------------------------------------------
Function GetIntByMoniker(list: TObjectList; aMoniker: IMoniker): Integer;
Var
  i: Integer;
Begin
  Result := -1;
  For i := 0 To list.Count - 1 Do
    With list[i] As TCapDeviceInfo Do
      If aMoniker = Moniker Then
      Begin
        Result := i;
        break;
      End;
End;

Function GetStringByMoniker(list: TObjectList; aMoniker: IMoniker): String;
Var
  i: Integer;
Begin
  Result := '';
  i := GetIntByMoniker(list, aMoniker);
  If (i >= 0) Then
    With list[i] As TCapDeviceInfo Do
      Result := DeviceNameInList;
End;

Function GetMonikerByString(list: TObjectList; aString: String): IMoniker;
Var
  i: Integer;
Begin
  Result := nil;
  For i := 0 To list.Count - 1 Do
    With list[i] As TCapDeviceInfo Do
      If aString = DeviceNameInList Then
      Begin
        Result := Moniker;
        break;
      End;
End;

//-----------------------------------------------------------------
Function FindDeviceNum(Const DeviceName: String; Devices: TObjectList): Integer;
Var
  i: Integer;
  d: TObject;
Begin
  Result := 1;
  For i := 0 To Devices.Count - 1 Do
  Begin
    d := Devices[i];
    If (d Is TCapDeviceInfo) And (TCapDeviceInfo(d).DeviceName = DeviceName) Then
      Result := Result + 1;
  End;
End;

Procedure EnumFilters(Const clsidDeviceClass: TGUID; Const FiltersList: TObjectList);
Var
  SysDevEnum: ICreateDevEnum;
  EnumCat: IEnumMoniker;
  Moniker: IMoniker;
  PropBag: IPropertyBag;
  varName: Olevariant;
  n: Integer;
  s: String;
Begin
  FiltersList.Clear;
  If (CoCreateInstance(CLSID_SystemDeviceEnum, nil, CLSCTX_INPROC, IID_ICreateDevEnum, SysDevEnum) = S_OK) Then
  Begin
    // enum available video capture devices
    If (SysDevEnum.CreateClassEnumerator(clsidDeviceClass, EnumCat, 0) = S_OK) Then
      While EnumCat.Next(1, Moniker, nil) = S_OK Do
      Begin
        If (Moniker.BindToStorage(nil, nil, IID_IPropertyBag, PropBag) = S_OK) And (PropBag <> nil) Then
        Begin
          PropBag.Read('FriendlyName', varName, nil);
          s := varName;
          If Trim(s) <> '' Then
          Begin
            n := FindDeviceNum(s, FiltersList);
            If n > 1 Then
              s := s + Format(' (%d)', [n]);
            FiltersList.Add(TCapDeviceInfo.Create(varName, s, Moniker));
          End;
        End;
      End;
  End;
End;

//-----------------------------------------------------------------
Var
  VideoDevicesList, AudioDevicesList, VideoCompressorsList, AudioCompressorsList: TObjectList;

Procedure BuildVideoDevicesList;
Begin
  EnumFilters(CLSID_VideoInputDeviceCategory, VideoDevicesList);
End;

Procedure BuildAudioDevicesList;
Begin
  EnumFilters(CLSID_AudioInputDeviceCategory, AudioDevicesList);
End;

Procedure BuildVideoCompressorsList;
Begin
  EnumFilters(CLSID_VideoCompressorCategory, VideoCompressorsList);
End;

Procedure BuildAudioCompressorsList;
Begin
  EnumFilters(CLSID_AudioCompressorCategory, AudioCompressorsList);
End;

Procedure BuildDevicesList;
Begin
  BuildVideoDevicesList;
  BuildAudioDevicesList;
  BuildVideoCompressorsList;
  BuildAudioCompressorsList;
End;

Procedure ClearDevicesList;
Begin
  VideoDevicesList.Clear;
  AudioDevicesList.Clear;
  VideoCompressorsList.Clear;
  AudioCompressorsList.Clear;
End;

//-----------------------------------------------------------------
// devices Enum functions
Function GetVideoDevicesList(Const Refresh: Boolean): TStringList;
Var
  i: Integer;
Begin
  If Refresh Or (VideoDevicesList.Count = 0) Then
    BuildVideoDevicesList;
  Result := TStringList.Create;
  Try
    For i := 0 To VideoDevicesList.Count - 1 Do
      Result.Add(TCapDeviceInfo(VideoDevicesList[i]).DeviceNameInList);
  Except
    Result.Free;
    Raise;
  End;
End;

Function GetAudioDevicesList(Const Refresh: Boolean): TStringList;
Var
  i: Integer;
Begin
  If Refresh Or (AudioDevicesList.Count = 0) Then
    BuildAudioDevicesList;
  Result := TStringList.Create;
  Try
    For i := 0 To AudioDevicesList.Count - 1 Do
      Result.Add(TCapDeviceInfo(AudioDevicesList[i]).DeviceNameInList);
  Except
    Result.Free;
    Raise;
  End;
End;

Function GetVideoCompressorsList(Const Refresh: Boolean = False): TStringList;
Var
  i: Integer;
Begin
  If Refresh Or (VideoCompressorsList.Count = 0) Then
    BuildVideoCompressorsList;
  Result := TStringList.Create;
  Try
    For i := 0 To VideoCompressorsList.Count - 1 Do
      Result.Add(TCapDeviceInfo(VideoCompressorsList[i]).DeviceNameInList);
  Except
    Result.Free;
    Raise;
  End;
End;

Function GetAudioCompressorsList(Const Refresh: Boolean = False): TStringList;
Var
  i: Integer;
Begin
  If Refresh Or (AudioCompressorsList.Count = 0) Then
    BuildAudioCompressorsList;
  Result := TStringList.Create;
  Try
    For i := 0 To AudioCompressorsList.Count - 1 Do
      Result.Add(TCapDeviceInfo(AudioCompressorsList[i]).DeviceNameInList);
  Except
    Result.Free;
    Raise;
  End;
End;

//-----------------------------------------------------------------
{ TSampleGrabberCB - ISampleGrabberCB realization}
Type
  TSampleGrabberCB = Class(TObject, ISampleGrabberCB)
  Private
    FBuffer: PByte;
    FBuffLen: Cardinal;
    FOwner: HWND;
    FEnable: Boolean;
    FLock: TCriticalSection;
  Public
    { IUnknown }
    Function QueryInterface(Const IID: TGUID; out Obj): HRESULT; Stdcall;
    Function _AddRef: Integer; Stdcall;
    Function _Release: Integer; Stdcall;
    { ISampleGrabberCB }
    Function SampleCB(SampleTime: Double; pSample: IMediaSample): HResult; Stdcall;
    Function BufferCB(SampleTime: Double; pBuffer: PByte; BufferLen: Longint): HResult; Stdcall;
  Public
    Property Owner: HWND Read FOwner Write FOwner;
    Property Buffer: PByte Read FBuffer;
    Property BufferLength: Cardinal Read FBuffLen;
    Property GrabbingEnabled: Boolean Read FEnable Write FEnable;

    Procedure ClearBuffer;
    // to avoid access to buffer from other thread
    // before accessing to buffer need to disable access to it
    // don't forget to enable access after using buffer data!
    Procedure DisableBufferAccess;
    Procedure EnableBufferAccess;

    Constructor Create;
    Destructor Destroy; Override;
  End;

{ TSampleGrabberCB }
Constructor TSampleGrabberCB.Create;
Begin
  FBuffer := nil;
  FBuffLen := 0;
  FLock := TCriticalSection.Create;
End;

Destructor TSampleGrabberCB.Destroy;
Begin
  ClearBuffer;
  FLock.Free;
  Inherited;
End;

Procedure TSampleGrabberCB.DisableBufferAccess;
Begin
  FLock.Enter;
End;

Procedure TSampleGrabberCB.EnableBufferAccess;
Begin
  FLock.Leave;
End;

Procedure TSampleGrabberCB.ClearBuffer;
Begin
  DisableBufferAccess;
  Try
    FBuffLen := 0;
    ReallocMem(FBuffer, FBuffLen);
  Finally
    EnableBufferAccess;
  End;
End;

{ TSampleGrabberCB - IUnknown }
Function TSampleGrabberCB._AddRef: Integer;
Begin
  Result := 2;
End;

Function TSampleGrabberCB._Release: Integer;
Begin
  Result := 1;
End;

Function TSampleGrabberCB.QueryInterface(Const IID: TGUID; out Obj): HRESULT;
Begin
  // We need to return the two event interfaces when they're asked for
  Result := E_NOINTERFACE;
  If CheckGUID(IID, ISampleGrabberCB) Or CheckGUID(IID, IUnknown) Then
  Begin
    If GetInterface(IID, Obj) Then
      Result := S_OK;
  End;
End;

{ TSampleGrabberCB - ISampleGrabberCB }
Function TSampleGrabberCB.SampleCB(SampleTime: Double; pSample: IMediaSample): HResult;
Begin // not implemented
  Result := S_OK;
End;

Function TSampleGrabberCB.BufferCB(SampleTime: Double; pBuffer: PByte; BufferLen: Integer): HResult;
Begin
  If GrabbingEnabled Then
  Begin
// Comment from microsoft programmer below
    // The sample grabber is calling us back on its deliver thread.
    // This is NOT the main app thread!

    //           !!!!! WARNING WARNING WARNING !!!!!

    // On Windows 9x systems, you are not allowed to call most of the
    // Windows API functions in this callback.  Why not?  Because the
    // video renderer might hold the global Win16 lock so that the video
    // surface can be locked while you copy its data.  This is not an
    // issue on Windows 2000, but is a limitation on Win95,98,98SE and ME.
    // Calling a 16-bit legacy function could lock the system, because
    // it would wait forever for the Win16 lock, which would be forever
    // held by the video renderer.
    // As a workaround, we will copy the bitmap data during the callback,
    // post a message to our app, and write the data later.
    GrabbingEnabled := False;
    DisableBufferAccess;
    Try
      ReallocMem(FBuffer, BufferLen);
      FBuffLen := BufferLen;
      CopyMemory(FBuffer, pBuffer, BufferLen);
    Finally
      EnableBufferAccess;
    End;
    // notify application frame buffer filled
    PostMessage(FOwner, WM_SAMPLECAPTURED, Integer(Self), 0);
  End;
  // notify application that next frame arrived
  PostMessage(FOwner, WM_SAMPLEPASSED, Integer(Self), 0);
  Result := S_OK;
End;

//-----------------------------------------------------------------
{ TVideoCapture }
Constructor TVideoCapture.Create(AOwner: TComponent);
Begin
  Inherited;
  CleanUp;
  FUseFrameRate := False;
  FWantAudio := True;
  FWantAudioPreview := True;
  FWantDVAudio := True;
  FWantPreview := True;
  FWantCapture := True;
  FWantBitmaps := True;
  Color  := clBlue;
  Width  := 100;
  Height := 100;
  FPreallocFileSize := 1;
  FPixelFormat := pf24bit;

  GrabberCB := TSampleGrabberCB.Create;

  FCaptureTimer := TTimer.Create(Self);
  FCaptureTimer.Interval := 100;
  FCaptureTimer.OnTimer := CaptureProgress;
End;

Destructor TVideoCapture.Destroy;
Begin
  StopPreview;
  StopCapture;
  TearDownGraph;
  CleanUp;

  FCaptureTimer.Enabled := False;
  FCaptureTimer.Free;
  GrabberCB.Free;

  // free resources occupied by lists
  ClearDevicesList;
  Inherited;
End;

//-----------------------------------------------------------------
Procedure TVideoCapture.ChooseDevices(nmVideo, nmAudio: IMoniker; ForceGraphRebuild: Boolean = False);
Begin
  If (ForceGraphRebuild) Or ((mVideo <> nmVideo) Or (mAudio <> nmAudio)) Then
  Begin
    mVideo := nmVideo;
    mAudio := nmAudio;
    TearDownGraph;
    Asm //this was added due to corrupted registers after func call (D5 compiler error?)
      Push Ebx;
      Push Edi;
      Push Esi
    End;
    InitCapFilters;
    Asm
      Pop Esi;
      Pop Edi;
      Pop Ebx
    End;

    If BuildGraph And WantPreview Then
      StartPreview;
    Try
      If Assigned(FOnChangeDevice) Then
        FOnChangeDevice(Self);
    Except
    End;
  End;
End;

Procedure TVideoCapture.ChooseCompressors(nmVComp, nmAComp: IMoniker; ForceGraphRebuild: Boolean);
Begin
  If (ForceGraphRebuild) Or ((mVComp <> nmVComp) Or (mAComp <> nmAComp)) Then
  Begin
    mVComp := nil;
    mAComp := nil;
    mVComp := nmVComp;
    mAComp := nmAComp;
    TearDownGraph;
    Asm //this was added due to corrupted registers after func call (D5 compiler error?)
      Push Ebx;
      Push Edi;
      Push Esi
    End;
    InitCapFilters;
    Asm
      Pop Esi;
      Pop Edi;
      Pop Ebx
    End;

    If BuildGraph And WantPreview Then
      StartPreview;
    Try
      If Assigned(FOnChangeCompressor) Then
        FOnChangeCompressor(Self);
    Except
    End;
  End;
End;

//-----------------------------------------------------------------
Procedure TVideoCapture.ChooseDevices(Const szVideo, szAudio: String; ForceGraphRebuild: Boolean = False);
Var
  nmVideo, nmAudio: IMoniker;
Begin
  nmVideo := nil;
  nmAudio := nil;

  If (VideoDevicesList.Count = 0) Then
    BuildVideoDevicesList;
  If (AudioDevicesList.Count = 0) Then
    BuildAudioDevicesList;
  nmVideo := GetMonikerByString(VideoDevicesList, szVideo);
  nmAudio := GetMonikerByString(AudioDevicesList, szAudio);

  ChooseDevices(nmVideo, nmAudio, ForceGraphRebuild);
  nmVideo := nil;
  nmAudio := nil;
End;

Procedure TVideoCapture.ChooseDevices(Const numVideo, numAudio: Integer; ForceGraphRebuild: Boolean = False);
Var
  nmVideo, nmAudio: IMoniker;
Begin
  nmVideo := nil;
  nmAudio := nil;

  If (numVideo >= 0) And (numVideo < VideoDevicesList.Count) Then
    nmVideo := TCapDeviceInfo(VideoDevicesList[numVideo]).Moniker;
  If (numAudio >= 0) And (numAudio < AudioDevicesList.Count) Then
    nmAudio := TCapDeviceInfo(AudioDevicesList[numAudio]).Moniker;

  ChooseDevices(nmVideo, nmAudio, ForceGraphRebuild);
  nmVideo := nil;
  nmAudio := nil;
End;

Procedure TVideoCapture.ChooseCompressors(Const szVComp, szAComp: String; ForceGraphRebuild: Boolean);
Var
  nmVComp, nmAComp: IMoniker;
Begin
  nmVComp := nil;
  nmAComp := nil;

  If (VideoCompressorsList.Count = 0) Then
    BuildVideoCompressorsList;
  If (AudioCompressorsList.Count = 0) Then
    BuildAudioCompressorsList;
  nmVComp := GetMonikerByString(VideoCompressorsList, szVComp);
  nmAComp := GetMonikerByString(AudioCompressorsList, szAComp);

  ChooseCompressors(nmVComp, nmAComp, ForceGraphRebuild);
  nmVComp := nil;
  nmAComp := nil;
End;

Procedure TVideoCapture.ChooseCompressors(Const numVComp, numAComp: Integer; ForceGraphRebuild: Boolean);
Var
  nmVComp, nmAComp: IMoniker;
Begin
  nmVComp := nil;
  nmAComp := nil;

  If (numVComp >= 0) And (numVComp < VideoCompressorsList.Count) Then
    nmVComp := TCapDeviceInfo(VideoCompressorsList[numVComp]).Moniker;
  If (numAComp >= 0) And (numAComp < AudioCompressorsList.Count) Then
    nmAComp := TCapDeviceInfo(AudioCompressorsList[numAComp]).Moniker;

  ChooseCompressors(nmVComp, nmAComp, ForceGraphRebuild);
  nmVComp := nil;
  nmAComp := nil;
End;

//-----------------------------------------------------------------
Procedure TVideoCapture.CleanUp;
Begin
  FreeCapFilters;

  VideoWindow := nil;
  MediaEvent  := nil;
  DroppedFrames := nil;

  Render := nil;
  Sink := nil;
  ConfigAviMux := nil;
  NullRenderer := nil;

  FCCAvail := False;
  FCapCC := False;
  FGraphBuilt := False;
  FCapturing := False;
  FPreviewing := False;
  FMasterStream := -1;
End;

Procedure TVideoCapture.FreeCapFilters;
Begin
  Graph := nil;
  Builder := nil;
  VCap  := nil;
  ACap  := nil;
  DVSplitter := nil;
  DVDec := nil;
  VComp := nil;
  AComp := nil;
  AStreamConf := nil;
  VStreamConf := nil;
  CaptureDialogs := nil;
  CompressDialogs := nil;
  Grabber := nil;
  FDialogs := [];
End;

Function TVideoCapture.MakeBuilder: Boolean;
Begin
  Result := (Builder <> nil) Or
    (CoCreateInstance(CLSID_CaptureGraphBuilder2, nil, CLSCTX_INPROC,
    IID_ICaptureGraphBuilder2, Builder) = NOERROR);
End;

Function TVideoCapture.MakeGraph: Boolean;
Begin
  Result := (Graph <> nil) Or
    (CoCreateInstance(CLSID_FilterGraph, nil, CLSCTX_INPROC,
    IID_IGraphBuilder, Graph) = NOERROR);
End;

Function TVideoCapture.Init: Boolean;
Begin
  // Create the filter graph and create the capture graph builder.
  Result := MakeGraph And MakeBuilder;
  If Not Result Then
    Exit;

  Builder.SetFiltergraph(Graph);
  BuildDevicesList;
  Result := (VideoDevicesList.Count > 0);// or (AudioDevicesList.Count>0);
End;

Function TVideoCapture.InitCapFilters: Boolean;
Var
  PropBag: IPropertyBag;
  hr:  HRESULT;
  varOle: Olevariant;
  Pin: IPin;
  AStream: IBaseFilter;
Begin
  FreeCapFilters;

  Result := MakeBuilder;
  If Not Result Then
  Begin
    ErrMsg(rsCantMakeGraphBuilder);
    Exit;
  End;

  DVSplitter := nil;
  DVDec := nil;

  Try
    VCap := nil;
    hr := 0;
    If mVideo <> nil Then
    Begin
      hr := mVideo.BindToStorage(nil, nil, IID_IPropertyBag, PropBag);
      If Succeeded(hr) Then
      Begin
        If PropBag.Read('FriendlyName', varOle, nil) = NOERROR Then
          FVCapFriendlyName := varOle;
        PropBag := nil;
      End;
      hr := mVideo.BindToObject(nil, nil, IID_IBaseFilter, VCap);
    End;
    If VCap = nil Then
      ErrMsgException(rsCantCreateVCaptureFilter, hr);

    // make a filtergraph, give it to the graph builder and put the video
    // capture filter in the graph
    If Not MakeGraph Then
      ErrMsgException(rsCantMakeGraph);

    If Builder.SetFiltergraph(Graph) <> NOERROR Then
      ErrMsgException(rsCantSetFilterGraph);

    If Graph.AddFilter(VCap, nil) <> NOERROR Then
      ErrMsgException(rsCantAddVFilterToGraph, hr);

    // !!! What if this interface isn't supported?
    // we use this interface to set the frame rate and get the capture size
    hr := Builder.FindInterface(@PIN_CATEGORY_CAPTURE, @MEDIATYPE_Interleaved,
      VCap, IID_IAMStreamConfig, VStreamConf);
    If hr <> NOERROR Then
    Begin
      hr := Builder.FindInterface(@PIN_CATEGORY_CAPTURE, @MEDIATYPE_Video,
        VCap, IID_IAMStreamConfig, VStreamConf);
      If hr <> NOERROR Then // this means we can't set frame rate (non-DV only)
        ErrMsg(rsCantFindVStreamConfig, hr);
    End;

    If IsDVSource Then
    Begin
      // insert DVSplitter & DVDecoder if they are not exist
      If Failed(Graph.FindFilterByName(DVDecoderFilterName, DVDec)) Then
      Begin
        If Failed(CoCreateInstance(CLSID_DVVideoCodec, nil, CLSCTX_INPROC, IID_IBaseFilter, DVDec)) Then
        Begin
          ErrMsgException(rsCantAddDVDecoder);
        End;
      End;
      If WantAudio And WantDVAudio Then
      Begin
        If Failed(Graph.FindFilterByName(DVSplitterFilterName, DVSplitter)) Then
        Begin
          If Failed(CoCreateInstance(CLSID_DVSplitter, nil, CLSCTX_INPROC, IID_IBaseFilter, DVSplitter)) Then
          Begin
            ErrMsgException(rsCantAddDVSplitter);
          End;
        End;
      End;
    End;
    // we use this interface to bring up the 3 dialogs
    // NOTE:  Only the VfW capture filter supports this.  This app only brings
    // up dialogs for legacy VfW capture drivers, since only those have dialogs
    Builder.FindInterface(@PIN_CATEGORY_CAPTURE, @MEDIATYPE_Video,
      VCap, IID_IAMVfwCaptureDialogs, CaptureDialogs);

    // add video compression filter
    If WantCapture And (VCap <> nil) And (mVComp <> nil) Then
    Begin
      hr := mVComp.BindToObject(nil, nil, IID_IBaseFilter, VComp);
      If (hr = S_OK) Then
      Begin
        Builder.FindInterface(@PIN_CATEGORY_CAPTURE, @MEDIATYPE_Video,
          VComp, IID_IAMVfwCompressDialogs, CompressDialogs);
      End;
    End;

    // there's no point making an audio capture filter
    If WantAudio Then
    Begin
      // create the audio capture filter, even if we are not capturing audio right
      // now, so we have all the filters around all the time.
      // We want an audio capture filter and some interfaces
      If (DVSplitter = nil) Then
      Begin // is not DV source
        If (mAudio <> nil) Then
        Begin
          hr := mAudio.BindToStorage(nil, nil, IID_IPropertyBag, PropBag);
          If Succeeded(hr) Then
          Begin
            If PropBag.Read('FriendlyName', varOle, nil) = NOERROR Then
              FACapFriendlyName := varOle;
            PropBag := nil;
          End;

          ACap := nil;
          mAudio.BindToObject(nil, nil, IID_IBaseFilter, ACap);
          If ACap = nil Then
            // there are no audio capture devices. We'll only allow video capture
            ErrMsg(rsCantMakeACapFilter)
          Else
          Begin
            // put the audio capture filter in the graph
            // We'll need this in the graph to get audio property pages
            If Graph.AddFilter(ACap, nil) <> NOERROR Then
              ErrMsgException(rsCantAddAFilterToGraph, hr);
          End;
        End;
        AStream := ACap;
      End
      Else
      Begin
        AStream := DVSplitter;
      End;

      If (ACap <> nil) Or (WantAudio And WantDVAudio) Then
      Begin
        // add audio compression filter
        If WantCapture And (mAComp <> nil) And Succeeded(mAComp.BindToObject(nil, nil, IID_IBaseFilter, AComp)) Then
        Begin
          AStream := AComp; // to control audio stream of AudioCompressor filter
        End;

        // !!! What if this interface isn't supported?
        // we use this interface to set the captured wave format
        hr := Builder.FindInterface(nil, @MEDIATYPE_Audio,
          AStream, IID_IAMStreamConfig, AStreamConf);
        If (hr <> NOERROR) And Not (IsDVSource And (AStream = DVSplitter)) Then
          ErrMsg(rsCantFindAStreamConfig);
      End;
    End;

    // Can this filter do closed captioning?
    hr := Builder.FindPin(VCap, PINDIR_OUTPUT, PIN_CATEGORY_VBI, TGUID(nil^),
      False, 0, Pin);
    If hr <> S_OK Then
      hr := Builder.FindPin(VCap, PINDIR_OUTPUT, PIN_CATEGORY_CC, TGUID(nil^),
        False, 0, Pin);
    FCCAvail := (hr = S_OK); // can't capture it, then
    If FCapCC Then
      Pin := nil;

    Result := True;
  Except
    FreeCapFilters;
    Result := False;
  End;
End;

Procedure TVideoCapture.ResizeWindow;
Begin
  If Assigned(VideoWindow) Then
    VideoWindow.SetWindowPosition(0, 0, ClientWidth, ClientHeight);
End;

Procedure TVideoCapture.SetSize(Var msg: TMessage);
Begin
  Inherited;
  ResizeWindow;
End;

Function TVideoCapture.FindVideoWindow: Boolean;
Begin
  // Get the preview window to be a child of our app's window
  // This will find the IVideoWindow interface on the renderer.  It is
  // important to ask the filtergraph for this interface... do NOT use
  // ICaptureGraphBuilder2::FindInterface, because the filtergraph needs to
  // know we own the window so it can give us display changed messages, etc.
  Result := (Graph.QueryInterface(IID_IVideoWindow, VideoWindow) = S_OK);
  If Result Then
  Begin
    VideoWindow.put_Owner(Handle);    // We own the window now
    VideoWindow.put_WindowStyle(WS_CHILD Or WS_CLIPSIBLINGS);    // you are now a child
    // give the preview window all our space
    ResizeWindow;
    VideoWindow.put_Visible(True);
  End;
End;

Procedure TVideoCapture.DoAborted(hr: HRESULT);
Begin
  StopCapture;
  If Assigned(FOnAborted) Then
    FOnAborted(Self);
End;

Procedure TVideoCapture.DoDeviceLost;
Begin
  If Assigned(FOnDeviceLost) Then
    FOnDeviceLost(Self);
  StopCapture;
  StopPreview;
End;

// graph event occured - get & process events
Procedure TVideoCapture.GraphEvent(Var msg: TMessage);
Var
  Event, param1, param2: Integer;
  wasCapturing, wasPreviewing: Boolean;
Begin
  wasCapturing  := Capturing;
  wasPreviewing := Previewing;
  If (MediaEvent <> nil) Then
  Begin
    While MediaEvent.GetEvent(Event, param1, param2, 0) = S_OK Do
    Begin
      Try
//  if you need parameters supplied by GetEvent function - process it here
//        case Event of
//        end;
      Finally
        MediaEvent.FreeEventParams(Event, param1, param2);
      End;
      Case Event Of
        EC_ERRORABORT:
          DoAborted(param1);
        EC_DEVICE_LOST:
          DoDeviceLost;
      End;
    End;
    // we have stopped capture now need to restore preview
    If ((Capturing <> wasCapturing) Or (wasPreviewing <> Previewing)) And (WantPreview) Then
    Begin
      StartPreview; // it will rebuild graph
    End;
  End;
End;

// we have to build video preview part of graph
// if we want to capture bitmaps need to render stream through grabber filter
Function TVideoCapture.RenderVideoPreview(Const VSource: IBaseFilter): Boolean;
Var
  FGrabber: IBaseFilter;
  mt:  TAM_Media_Type;
  flt: IBaseFilter;
Begin
  flt := VSource;
  If WantBitmaps Then
  Begin
    Result := Succeeded(Graph.FindFilterByName(SampleGrabberFilterName, FGrabber)) Or // already exists
      ( // have to make new one
      (CoCreateInstance(CLSID_SampleGrabber, nil, CLSCTX_INPROC, IID_IBaseFilter, FGrabber) = NOERROR) And
      (Graph.AddFilter(FGrabber, SampleGrabberFilterName) = S_OK)
      );

    Result := Result And (FGrabber.QueryInterface(IID_ISampleGrabber, Grabber) = S_OK);

    If Result Then
    Begin
      If (FPixelFormat <> pfDevice) Then
      Begin
        // set prefered bitmap pixel depth
        ZeroMemory(@mt, sizeof(mt));
        mt.formattype := FORMAT_VideoInfo;
        mt.majortype := MEDIATYPE_Video;
        mt.subtype := PixelFormatGUIDs[FPixelFormat]^;
        Grabber.SetMediaType(mt);
      End;

      If Succeeded(Builder.RenderStream(nil, @MEDIATYPE_Video, flt, nil, FGrabber)) Then
      Begin
        Grabber.SetBufferSamples(False);
        Grabber.SetOneShot(False);
        TSampleGrabberCB(GrabberCB).Owner := Handle;
        Grabber.SetCallback(TSampleGrabberCB(GrabberCB), 1);
        flt := FGrabber;
      End
      Else
      Begin
        // we cant connect grabber - probably wrong mediatype?
        // TO DO: try to request DecoderOut available mediatypes and
        //        set one of them
        ErrMsg(rsCantInstallSampleGrabber);
      End;

    End;
  End;
  Result := Succeeded(Builder.RenderStream(nil, @MEDIATYPE_Video, flt, nil, nil));
End;

// Tear down everything downstream of a given filter
Procedure TVideoCapture.NukeDownstream(pf: IBaseFilter);
Var
  pP, pTo: IPin;
  pins: IEnumPins;
  pininfo: TPIN_INFO;
  hr: HRESULT;
Begin
  pins := nil;
  hr := pf.EnumPins(pins);
  pins.Reset;
  While hr = NOERROR Do
  Begin
    hr := pins.Next(1, pP, nil);
    If (hr = S_OK) And (pP <> nil) Then
    Begin
      pP.ConnectedTo(pTo);
      If pTo <> nil Then
      Begin
        hr := pTo.QueryPinInfo(pininfo);
        If hr = NOERROR Then
        Begin
          If pininfo.dir = PINDIR_INPUT Then
          Begin
            NukeDownstream(pininfo.pFilter);
            Graph.Disconnect(pTo);
            Graph.Disconnect(pP);
            Graph.RemoveFilter(pininfo.pFilter);
          End;
          pininfo.pFilter := nil;
        End;
        pTo := nil;
      End;
      pP := nil;
    End;
  End;
  pins := nil;
End;

// Tear down everything downstream of the capture filters, so we can build
// a different capture graph.  Notice that we never destroy the capture filters
// and WDM filters upstream of them, because then all the capture settings
// we've set would be lost.
Procedure TVideoCapture.TearDownGraph;
Begin
  If Not FGraphBuilt Then
    Exit;
  If Capturing Then
    StopCapture;
  If Previewing Then
    StopPreview;

  If VideoWindow <> nil Then
  Begin
    // stop drawing in our window, or we may get wierd repaint effects
    VideoWindow.put_Visible(False);
    VideoWindow.put_Owner(0);
  End;
  // free compressors and other filters we used
  VideoWindow := nil;
  Sink := nil;
  NullRenderer := nil;
  ConfigAviMux := nil;
  Render := nil;
  Grabber := nil;
  MediaEvent := nil;
  DroppedFrames := nil;

  // destroy the graph downstream of our capture filters
  If VCap <> nil Then
    NukeDownstream(VCap);
  If ACap <> nil Then
    NukeDownstream(ACap);

  FGraphBuilt := False;
  If (WantCapture) Then
    DeleteFile(TempCaptureFileName);
End;

// build the preview graph!
// !!! PLEASE NOTE !!!  Some new WDM devices have totally separate capture
// and preview settings.  An application that wishes to preview and then
// capture may have to set the preview pin format using IAMStreamConfig on the
// preview pin, and then again on the capture pin to capture with that format.
// In this sample app, there is a separate page to set the settings on the
// capture pin and one for the preview pin.  To avoid the user
// having to enter the same settings in 2 dialog boxes, an app can have its own
// UI for choosing a format (the possible formats can be enumerated using
// IAMStreamConfig) and then the app can programmatically call IAMStreamConfig
// to set the format on both pins.

// build the capture graph!
Function TVideoCapture.BuildGraph: Boolean;
Var
  hr:  HRESULT;
  pmt: PAM_MEDIA_TYPE;
  SmartTee: IBaseFilter;
  InfiniteTee: IBaseFilter;
  VSource, ASource: IBaseFilter; // aliases to video/audio sources
  Sink2: IFileSinkFilter2;
Begin
  // we have no GraphBuilder((
  Result := Assigned(Graph);
  If Not Result Then
    Exit;
  // we have one already
  Result := FGraphBuilt;
  If Result Then
    Exit;
  // Result = false

  // We don't have necessary capture filters
  If (VCap = nil) Then
    Exit;
  If (WantAudio) And ((ACap = nil) And (Not (IsDVSource And WantDVAudio))) Then
    Exit;

  If (WantCapture) Then
  Begin
    // no capture file name yet... we need one first
    If UseTempFile Then
      FCaptureFile := TempCaptureFileName
    Else
      FCaptureFile := CaptureFileName;

    If (FCaptureFile = '') Then
    Begin
      ErrMsg(rsEmptyFileName);
      Exit;
    End;

    DeleteFile(FCaptureFile);

    If FDoPreallocFile And Not AllocCaptureFile(PreallocFileSize) Then
    Begin
      ErrMsg(rsFailedToAllocFileSize);
      Exit;
    End;
  End;

  Try
    VSource := VCap;
    ASource := ACap;

    If IsDVSource Then
    Begin
      If (DVDec <> nil) Then
        Graph.AddFilter(DVDec, DVDecoderFilterName);
      If (DVSplitter <> nil) Then
        Graph.AddFilter(DVSplitter, DVSplitterFilterName);
      If (DVDec <> nil) And (Builder.RenderStream(nil, @MEDIATYPE_Interleaved, VCap, DVSplitter, DVDec) = S_OK) Then
      Begin
        SetDVResolution(FDVResolution);
        VSource := DVDec;
        If WantAudio And WantDVAudio Then
          ASource := DVSplitter;
      End
      Else
        ErrMsgException(rsCantRenderVCaptureStream);
    End;

    // insert Smart Tee Filter
    If Failed(CoCreateInstance(CLSID_SmartTee, nil, CLSCTX_INPROC, IID_IBaseFilter, SmartTee)) Or
      Failed(Graph.AddFilter(SmartTee, SmartTeeFilterName)) Or
      Failed(Builder.RenderStream(nil, @MEDIATYPE_Video, VSource, nil, SmartTee)) Then
    Begin
      ErrMsgException(rsCantAddSmartTee);
    End;
    VSource := SmartTee;

    // Infinite Tee Filter
    If (ASource <> nil) And (WantAudioPreview) Then
    Begin
      If Failed(CoCreateInstance(CLSID_InfTee, nil, CLSCTX_INPROC, IID_IBaseFilter, InfiniteTee)) Or
        Failed(Graph.AddFilter(InfiniteTee, InfTeeFilterName)) Or
        Failed(Builder.RenderStream(nil, @MEDIATYPE_Audio, ASource, nil, InfiniteTee)) Then
      Begin
        ErrMsgException(rsCantAddInfTee);
      End;
      ASource := InfiniteTee;
    End;

    // Rendering audio preview stream
    If WantPreview And WantAudioPreview And (ASource <> nil) Then
    Begin
      If Failed(Builder.RenderStream(nil, @MEDIATYPE_Audio, ASource, nil, nil)) Then
        ErrMsgException(rsCantRenderAPreviewStream);
    End;

    If WantCapture Then
    Begin
//      if Succeeded(CoCreateInstance(CLSID_NullRenderer, nil, CLSCTX_INPROC, IID_IBaseFilter, NullRenderer)) then begin
//        Graph.AddFilter(NullRenderer, NullRendererFilterName);
//      end;
      // We need a rendering section that will write the capture file out in AVI file format
      hr := Builder.SetOutputFileName(MEDIASUBTYPE_Avi, PWCHAR(FCaptureFile), Render, Sink);
      If (hr <> NOERROR) Then
        ErrMsgException(rsCantSetCaptureFile);
      If Succeeded(Sink.QueryInterface(IID_IFileSinkFilter2, Sink2)) Then
      Begin
        // set overwrite mode for file sinker
        Sink2.SetMode(AM_FILE_OVERWRITE);
        Sink2 := nil;
      End;
      // Now tell the AVIMUX to write out AVI files that old apps can read properly.
      // If we don't, most apps won't be able to tell where the keyframes are, slowing down editing considerably
      // Doing this will cause one seek (over the area the index will go) when
      // you capture past 1 Gig, but that's no big deal.
      // NOTE: This is on by default, so it's not necessary to turn it on
      hr := Render.QueryInterface(IID_IConfigAviMux, ConfigAviMux);
      If (hr = NOERROR) And (ConfigAviMux <> nil) Then
      Begin
        ConfigAviMux.SetOutputCompatibilityIndex(True);
        If (ASource <> nil) Then // Also, set the proper MASTER STREAM
          MasterStream := FMasterStream;
      End;

      If (VComp <> nil) Then
        Graph.AddFilter(VComp, VCompFilterName);
      hr := Builder.RenderStream(nil, @MEDIATYPE_Video, VSource, VComp, Render);
      If (hr <> NOERROR) Then
        ErrMsgException(rsCantRenderVCaptureStream);

      If ASource <> nil Then
      Begin
        // render audio capture stream
        If (AComp <> nil) Then
          Graph.AddFilter(AComp, ACompFilterName);
        hr := Builder.RenderStream(nil, @MEDIATYPE_Audio, ASource, AComp, Render);
        If Failed(hr) Then
          ErrMsgException(rsCantRenderACaptureStream);
      End;
    End;

    // Rendering video preview stream
    If WantPreview And Not RenderVideoPreview(VSource) Then
      ErrMsgException(rsCantRenderPreviewStream);

    // NOTE: We do this even if we didn't ask for a preview, because rendering
    // the capture pin may have rendered the preview pin too (WDM overlay
    // devices) because they must have a preview going.  So we better always
    // put the preview window in our app, or we may get a top level window
    // appearing out of nowhere!
    If Not FindVideoWindow And (FWantPreview) Then
      ErrMsg(rsThisGraphCantPreview);

    // Render the closed captioning pin? It could be a CC or a VBI category pin,
    // depending on the capture driver
    If (FCCAvail And FCapCC) Then
    Begin
      hr := Builder.RenderStream(@PIN_CATEGORY_CC, nil, VCap, nil, Render);
      If (hr <> NOERROR) Then
      Begin
        hr := Builder.RenderStream(@PIN_CATEGORY_VBI, nil, VCap, nil, Render);
        If (hr <> NOERROR) Then
          ErrMsg(rsCantRenderCC);
      End;
      // To preview and capture VBI at the same time, we can call this twice
      If (WantPreview) Then
        Builder.RenderStream(@PIN_CATEGORY_VBI, nil, VCap, nil, nil);
    End;

    // now tell it what frame rate to capture at.  Just find the format it
    // is capturing with, and leave everything alone but change the frame rate
    If UseFrameRate Then
      hr := E_FAIL
    Else
      hr := NOERROR;

    If (VStreamConf <> nil) And UseFrameRate Then
    Begin
  // DV capture does not use a VIDEOINFOHEADER
      If VStreamConf.GetFormat(pmt) = NOERROR Then
      Begin
        Try
          If CheckGuid(pmt^.formattype, FORMAT_VideoInfo) Then
          Begin
            PVIDEOINFOHEADER(pmt^.pbFormat)^.AvgTimePerFrame := round(10000000 / FrameRate);
            hr := VStreamConf.SetFormat(pmt^);
            If hr <> NOERROR Then
              ErrMsg(rsCantSetPreviewFrameRate, hr);
          End;
        Finally
          DeleteMediaType(pmt);
        End;
      End;
    End;
    If (hr <> NOERROR) Then
      ErrMsg(rsCantSetCaptureFrameRate);

    // now ask the filtergraph to tell us when something is completed or aborted
    // (EC_COMPLETE, EC_USERABORT, EC_ERRORABORT).  This is how we will find out
    // if the disk gets full while capturing
    If (Graph.QueryInterface(IID_IMediaEventEx, MediaEvent) = NOERROR) Then
      MediaEvent.SetNotifyWindow(Handle, WM_FGNOTIFY, 0);

// All done.
    GetAvailableDialogs;
    FGraphBuilt := True;
    Result := True;
  Except
    TearDownGraph;
    Result := False;
  End;

End;

Function TVideoCapture.GetIsDVSource: Boolean;
Var
  pmt: PAM_Media_Type;
Begin
  Result := False;
  If (VStreamConf <> nil) And (VStreamConf.GetFormat(pmt) = S_OK) Then
    Try
      Result := CheckGUID(pmt^.majortype, MEDIATYPE_Interleaved);
    Finally
      DeleteMediaType(pmt);
    End;
End;

//--------------------------------------------------------------------
// current capture and compressors devices
Function TVideoCapture.GetACapSourceIdx: Integer;
Begin
  Result := GetIntByMoniker(AudioDevicesList, mAudio);
  If Result < 0 Then
  Begin
    BuildAudioDevicesList;
    Result := GetIntByMoniker(AudioDevicesList, mAudio);
  End;
End;

Function TVideoCapture.GetACompDeviceIdx: Integer;
Begin
  Result := GetIntByMoniker(AudioCompressorsList, mAComp);
  If Result < 0 Then
  Begin
    BuildAudioCompressorsList;
    Result := GetIntByMoniker(AudioCompressorsList, mAComp);
  End;
End;

Function TVideoCapture.GetVCapSourceIdx: Integer;
Begin
  Result := GetIntByMoniker(VideoDevicesList, mVideo);
  If Result < 0 Then
  Begin
    BuildVideoDevicesList;
    Result := GetIntByMoniker(VideoDevicesList, mAComp);
  End;
End;

Function TVideoCapture.GetVCompDeviceIdx: Integer;
Begin
  Result := GetIntByMoniker(VideoCompressorsList, mVComp);
  If Result < 0 Then
  Begin
    BuildVideoCompressorsList;
    Result := GetIntByMoniker(VideoCompressorsList, mAComp);
  End;
End;

Function TVideoCapture.GetACapSource: String;
Begin
  Result := GetStringByMoniker(AudioDevicesList, mAudio);
  If Result = '' Then
  Begin
    BuildAudioDevicesList;
    Result := GetStringByMoniker(AudioDevicesList, mAudio);
  End;
End;

Function TVideoCapture.GetACompDevice: String;
Begin
  Result := GetStringByMoniker(AudioCompressorsList, mAComp);
  If Result = '' Then
  Begin
    BuildAudioCompressorsList;
    Result := GetStringByMoniker(AudioCompressorsList, mAudio);
  End;
End;

Function TVideoCapture.GetVCapSource: String;
Begin
  Result := GetStringByMoniker(VideoDevicesList, mVideo);
  If Result = '' Then
  Begin
    BuildVideoDevicesList;
    Result := GetStringByMoniker(VideoDevicesList, mAudio);
  End;
End;

Function TVideoCapture.GetVCompDevice: String;
Begin
  Result := GetStringByMoniker(VideoCompressorsList, mVComp);
  If Result = '' Then
  Begin
    BuildVideoCompressorsList;
    Result := GetStringByMoniker(VideoCompressorsList, mAudio);
  End;
End;

//--------------------------------------------------------------------
// working with avaiable modes of current capture filters
Function TVideoCapture.GetACapModeCount: Integer;
Var
  capCount, capSize: Integer;
Begin
  Result := 0;
  If (AStreamConf <> nil) And (AStreamConf.GetNumberOfCapabilities(capCount, capSize) = S_OK) Then
  Begin
    Result := capCount;
  End;
End;

Function TVideoCapture.GetACapMode: TACapMode;
Var
  pmt: PAM_MEDIA_TYPE;
Begin
  ZeroMemory(@Result, SizeOf(Result));
  If (AStreamConf <> nil) And (AStreamConf.GetFormat(pmt) = S_OK) Then
    Try
      If CheckGUID(pmt^.formattype, FORMAT_WaveFormatEx) Then
      Begin
        Result.MediaType := pmt^.majortype;
        Result.MediaSubType := pmt^.subtype;
        Result.SampleRate := PWAVEFORMATEX(pmt^.pbFormat)^.nSamplesPerSec;
        Result.SampleSize := PWAVEFORMATEX(pmt^.pbFormat)^.wBitsPerSample;
        Result.Channels := PWAVEFORMATEX(pmt^.pbFormat)^.nChannels;
        Result.AvgSampleRate := PWAVEFORMATEX(pmt^.pbFormat)^.nAvgBytesPerSec;
      End;
    Finally
      DeleteMediaType(pmt);
    End;
End;

Function TVideoCapture.GetACapModes(i: Integer): TACapMode;
Var
  pmt: PAM_MEDIA_TYPE;
  aConfig: TAudio_Stream_Config_Caps;
Begin
  ZeroMemory(@Result, SizeOf(Result));
  If (i >= 0) And (i < ACapModeCount) And
    (AStreamConf <> nil) And (AStreamConf.GetStreamCaps(i, pmt, aConfig) = S_OK) Then
    Try
      If CheckGUID(pmt^.formattype, FORMAT_WaveFormatEx) Then
      Begin
        Result.MediaType := pmt^.majortype;
        Result.MediaSubType := pmt^.subtype;
        Result.SampleRate := PWAVEFORMATEX(pmt^.pbFormat)^.nSamplesPerSec;
        Result.SampleSize := PWAVEFORMATEX(pmt^.pbFormat)^.wBitsPerSample;
        Result.Channels := PWAVEFORMATEX(pmt^.pbFormat)^.nChannels;
        Result.AvgSampleRate := PWAVEFORMATEX(pmt^.pbFormat)^.nAvgBytesPerSec;
      End;
    Finally
      DeleteMediaType(pmt);
    End;
End;

Function TVideoCapture.SetACapMode(i: Integer): Boolean;
Var
  pmt: PAM_MEDIA_TYPE;
  aConfig: TAudio_Stream_Config_Caps;
  wasPreviewing: Boolean;
Begin
  Result := False;
  If Capturing Then
    Exit;
  wasPreviewing := Previewing;
  If (i >= 0) And (i < ACapModeCount) And
    (AStreamConf <> nil) And (AStreamConf.GetStreamCaps(i, pmt, aConfig) = S_OK) Then
  Begin
    Try
      // we need to stop streaming first and then rerender stream
      StopPreview();
      Result := AStreamConf.SetFormat(pmt^) = S_OK;
    Finally
      DeleteMediaType(pmt);
    End;
  End;
  If wasPreviewing Then
    StartPreview();
  If Result And Assigned(FOnChangeACapMode) Then
    FOnChangeACapMode(Self);
End;

Function TVideoCapture.SetACapMode(Const mode: TACapMode): Boolean;
Var
  i, capCount: Integer;
Begin
  Result := IsEqualModes(mode, ACapMode);
  If Not Result Then
  Begin
    capCount := ACapModeCount;
    For i := 0 To capCount - 1 Do
      If IsEqualModes(mode, ACapModes[i]) Then
      Begin
        Result := SetACapMode(i);
        break;
      End;
  End;
End;

Function TVideoCapture.GetVCapModeCount: Integer;
Var
  capCount, capSize: Integer;
Begin
  Result := 0;
  If Not IsDVSource Then
  Begin
    If (VStreamConf <> nil) And (VStreamConf.GetNumberOfCapabilities(capCount, capSize) = S_OK) Then
    Begin
      Result := capCount;
    End;
  End
  Else
  Begin
    Result := DVCapModesCount;
  End;
End;

Function TVideoCapture.GetVCapMode: TVCapMode;
Var
  pmt: PAM_MEDIA_TYPE;
  DVDecProp: IIPDVDec;
  _DVRes: Integer;
Begin
  ZeroMemory(@Result, SizeOf(Result));
  If Not IsDVSource Then
  Begin
    If (VStreamConf <> nil) And (VStreamConf.GetFormat(pmt) = S_OK) Then
      Try
        Result.MediaType := pmt^.majortype;
        Result.MediaSubType := pmt^.subtype;
        If CheckGUID(pmt^.formattype, FORMAT_VideoInfo) Then
        Begin
          Result.Width  := ABS(PVIDEOINFOHEADER(pmt^.pbFormat)^.bmiHeader.biWidth);
          Result.Height := ABS(PVIDEOINFOHEADER(pmt^.pbFormat)^.bmiHeader.biHeight);
          Result.BitCount := PVIDEOINFOHEADER(pmt^.pbFormat)^.bmiHeader.biBitCount;
        End
        Else
        If CheckGUID(pmt^.formattype, FORMAT_VideoInfo2) Then
        Begin
          Result.Width  := ABS(PVIDEOINFOHEADER2(pmt^.pbFormat)^.bmiHeader.biWidth);
          Result.Height := ABS(PVIDEOINFOHEADER2(pmt^.pbFormat)^.bmiHeader.biHeight);
          Result.BitCount := PVIDEOINFOHEADER2(pmt^.pbFormat)^.bmiHeader.biBitCount;
        End
      Finally
        DeleteMediaType(pmt);
      End;
  End
  Else
  Begin
    If Graph = nil Then
      Exit;
    // get DV window size
    If (DVDec <> nil) And
      (DVDec.QueryInterface(IIPDVDec, DVDecProp) = S_OK) And
      (DVDecProp.get_IPDisplay(_DVRes) = S_OK) And
      (_DVRes >= Low(DVRes)) And (_DVRes <= High(DVRes)) Then
      With DVRes[_DVRes] Do
      Begin
        Result.Width  := DVWidth;
        Result.Height := DVHeight;
        Result.BitCount := DVBits;
      End;
  End;
End;

Function TVideoCapture.GetVCapModes(i: Integer): TVCapMode;
Var
  pmt: PAM_MEDIA_TYPE;
  vConfig: TVideo_Stream_Config_Caps;
Begin
  ZeroMemory(@Result, SizeOf(Result));
  If Not IsDVSource Then
  Begin
    If (i >= 0) And (i < VCapModeCount) And
      (VStreamConf <> nil) And (VStreamConf.GetStreamCaps(i, pmt, vConfig) = S_OK) Then
    Begin
      Try
        Result.MediaType := pmt^.majortype;
        Result.MediaSubType := pmt^.subtype;
        If CheckGUID(pmt^.formattype, FORMAT_VideoInfo) Then
        Begin
          Result.Width  := ABS(PVIDEOINFOHEADER(pmt^.pbFormat)^.bmiHeader.biWidth);
          Result.Height := ABS(PVIDEOINFOHEADER(pmt^.pbFormat)^.bmiHeader.biHeight);
          Result.BitCount := PVIDEOINFOHEADER(pmt^.pbFormat)^.bmiHeader.biBitCount;
        End
        Else
        If CheckGUID(pmt^.formattype, FORMAT_VideoInfo2) Then
        Begin
          Result.Width  := ABS(PVIDEOINFOHEADER2(pmt^.pbFormat)^.bmiHeader.biWidth);
          Result.Height := ABS(PVIDEOINFOHEADER2(pmt^.pbFormat)^.bmiHeader.biHeight);
          Result.BitCount := PVIDEOINFOHEADER2(pmt^.pbFormat)^.bmiHeader.biBitCount;
        End;
        If (PVIDEOINFOHEADER(pmt^.pbFormat)^.AvgTimePerFrame > 0) Then
          Result.FrameRate := 10000000 / PVIDEOINFOHEADER(pmt^.pbFormat)^.AvgTimePerFrame; //??
      Finally
        DeleteMediaType(pmt);
      End;
      If (vConfig.MaxFrameInterval > 0) Then
        Result.MinFrameRate := 10000000 / vConfig.MaxFrameInterval; //??
      If (vConfig.MinFrameInterval > 0) Then
        Result.MaxFrameRate := 10000000 / vConfig.MinFrameInterval; //??
    End;
  End
  Else
  Begin
    With DVRes[i + Low(DVRes)] Do
    Begin
      Result.Width  := DVWidth;
      Result.Height := DVHeight;
      Result.BitCount := DVBits;
    End;
  End;
End;

Function TVideoCapture.SetVCapMode(i: Integer): Boolean;
Var
  pmt: PAM_MEDIA_TYPE;
  vConfig: TVideo_Stream_Config_Caps;
//  wasPreviewing: boolean;
//  GraphWasBuilt: boolean;
Begin
  Result := False;
  If Capturing Then
    Exit;  // dont let change format during capture
//  wasPreviewing:= Previewing;
//  GraphWasBuilt:= FGraphBuilt;
  TearDownGraph;
  If Not IsDVSource Then
  Begin
    If (i >= 0) And (i < VCapModeCount) And
      (VStreamConf <> nil) And (VStreamConf.GetStreamCaps(i, pmt, vConfig) = S_OK) Then
    Begin
      Try
        // we need to stop streaming first and then rerender stream
        Result := VStreamConf.SetFormat(pmt^) = S_OK;
      Finally
        DeleteMediaType(pmt);
      End;
    End;
  End
  Else
  Begin
    // set DV window size
    FDVResolution := DVRes1[i + DVRESOLUTION_FULL];
  End;
  If Result And Assigned(FOnChangeVCapMode) Then
    FOnChangeVCapMode(Self);
//  if GraphWasBuilt and BuildGraph then begin
//    if wasPreviewing then StartPreview;
//  end;
End;

Function TVideoCapture.SetVCapMode(Const mode: TVCapMode): Boolean;
Var
  i, capCount: Integer;
Begin
  Result := IsEqualModes(mode, VCapMode);
  If Not Result Then
  Begin
    capCount := VCapModeCount;
    For i := 0 To capCount - 1 Do
      If IsEqualModes(mode, VCapModes[i]) Then
      Begin
        Result := SetVCapMode(i);
        break;
      End;
  End;
End;

Function TVideoCapture.GetACapModeIdx: Integer;
Var
  curMode: TACapMode;
  i, capCount: Integer;
Begin
  Result  := -1;
  curMode := ACapMode;
  capCount := ACapModeCount;
  For i := 0 To capCount - 1 Do
    If IsEqualModes(curMode, ACapModes[i]) Then
    Begin
      Result := i;
      break;
    End;
End;

Function TVideoCapture.GetVCapModeIdx: Integer;
Var
  curMode: TVCapMode;
  i, capCount: Integer;
Begin
  Result  := -1;
  curMode := VCapMode;
  capCount := VCapModeCount;
  For i := 0 To capCount - 1 Do
    If IsEqualModes(curMode, VCapModes[i]) Then
    Begin
      Result := i;
      break;
    End;
End;

Procedure TVideoCapture.SetDVResolution(Const Value: TDVResolution);
Var
  DVDecProp: IIPDVDec;
//  wasPreviewing: boolean;
//  wasGraphBuilt: boolean;
Begin
  If Capturing Then
    Exit;
  FDVResolution := Value;
  If Graph = nil Then
    Exit;
  If IsDVSource And (Value <> dvrDontWorry) Then
  Begin
//    wasGraphBuilt:= FGraphBuilt;
//    wasPreviewing:= Previewing;
    TearDownGraph;
    // set DV window size
    If (DVDec <> nil) And (DVDec.QueryInterface(IIPDVDec, DVDecProp) = S_OK) Then
    Begin
      DVDecProp.put_IPDisplay(DVResolutions[Value]);
    End;
//    if wasGraphBuilt then begin
//      if BuildGraph and wasPreviewing then StartPreview;
//   end;
  End;
End;

//--------------------------------------------------------------------
Procedure TVideoCapture.SetMasterStream(Const Value: Integer);
Begin
  If (ConfigAviMux <> nil) And (ConfigAviMux.SetMasterStream(Value) = NOERROR) Then
    FMasterStream := Value
  Else
    ErrMsg(rsSetMasterStreamFailed);
End;

// Single frame captured. Called in main application thread.
Procedure TVideoCapture.BitmapGrabbed(Var msg: TWMSampleCaptured);
Var
  MediaType: TAM_Media_Type;
  bih:  TBitmapInfoHeader;
  BitmapInfo: TBITMAPINFO;
  w, h: Integer;
  usage: Cardinal;
  CapturedImage: TCapturedBitmap;
  SampleGrabberCB: TSampleGrabberCB;
  BitmapMade: Boolean;
Begin
  Try
    // process only own grabber messages
    If (Not (msg.GrabberCB Is TSampleGrabberCB)) Or (msg.GrabberCB <> GrabberCB) Then
      Exit;
    SampleGrabberCB := TSampleGrabberCB(msg.GrabberCB);
    If (Grabber.GetConnectedMediaType(MediaType) = S_OK) Then
    Begin
      Try
        // Get a pointer to the video header.
        If CheckGUID(MediaType.FormatType, FORMAT_VideoInfo) Then
          bih := PVideoInfoHeader(mediatype.pbFormat)^.bmiHeader
        Else
        If CheckGUID(MediaType.FormatType, FORMAT_VideoInfo2) Then
          bih := PVideoInfoHeader2(mediatype.pbFormat)^.bmiHeader
        Else
          Abort;
        // The video header contains the bitmap information.
        // Copy it into a BITMAPINFO structure.
        ZeroMemory(@BitmapInfo, sizeof(BitmapInfo));
        Move(bih, BitmapInfo.bmiHeader, sizeof(bih));
      Finally
        FreeMediaType(MediaType);
      End;
      // Copy the image into the buffer.
      w := BitmapInfo.bmiHeader.biWidth;
      h := BitmapInfo.bmiHeader.biHeight;
      CapturedImage := TCapturedBitmap.Create;
      Try
        Case BitmapInfo.bmiHeader.biBitCount Of
          1:
            CapturedImage.PixelFormat := pf1bit;
          4:
            CapturedImage.PixelFormat := pf4bit;
          8:
            CapturedImage.PixelFormat := pf8bit;
          15:
            CapturedImage.PixelFormat := pf15bit;
          16:
            CapturedImage.PixelFormat := pf16bit;
          24:
            CapturedImage.PixelFormat := pf24bit;
          32:
            CapturedImage.PixelFormat := pf32bit;
          Else
          // strange color depth - let bitmap be unchanged
        End;
        CapturedImage.Width  := w;  // enlarge image to ensure the whole DIB gets copied
        CapturedImage.Height := h;

        If (BitmapInfo.bmiHeader.biClrUsed = 0) Then
          usage := DIB_RGB_COLORS  // BITMAP contains real color values
        Else
          usage := DIB_PAL_COLORS; // BITMAP uses pallete

        If (SampleGrabberCB.BufferLength > 0) Then
        Begin
          SampleGrabberCB.DisableBufferAccess; // to avoid simultaneous access to buffer
          Try
            BitmapMade := StretchDIBits(CapturedImage.Canvas.Handle, // handle of device context
              0, 0, w, h,                  // destination rectangle
              0, 0, w, h,                  // source rectangle
              SampleGrabberCB.Buffer,      // address of bitmap bits
              BitmapInfo,            // address of bitmap data
              usage, SRCCOPY) <> 0;
          Finally
            SampleGrabberCB.EnableBufferAccess;
          End;
          If BitmapMade And Assigned(FOnBitmapGrabbed) Then
            FOnBitmapGrabbed(CapturedImage);
        End;
      Finally
        CapturedImage.Free;
      End;
      // SampleGrabberCB.ClearBuffer; // don't do it to avoid reallocating buffer next time
    End;
  Except
  End;
End;

Function TVideoCapture.CaptureFrame: Boolean;
Begin
  Result := (Grabber <> nil) And FWantBitmaps And (Previewing Or Capturing);
  TSampleGrabberCB(GrabberCB).GrabbingEnabled := Result;
End;

Procedure TVideoCapture.FramePassed(Var msg: TMessage);
Begin
  If Assigned(FOnFrameRendered) Then
    FOnFrameRendered(Self);
End;

//--------------------------------------------------------------------
Function ControlPin(pin: IPin; start: bool; Const SendExtraFrame: bool = False): Boolean;
Var
  CapCtrl: IAMStreamControl;
Begin
  Result := False;
  If Succeeded(pin.QueryInterface(IID_IAMStreamControl, CapCtrl)) Then
  Begin
    If start Then
      Result := Succeeded(CapCtrl.StartAt(nil, 0))
    Else
      Result := Succeeded(CapCtrl.StopAt(nil, SendExtraFrame, 0));
  End;
//  Graph.Reconnect(pin);
End;

Function ControlFilter(Filter: IBaseFilter; start: bool): Boolean;
Var
  pins: IEnumPins;
  pin: IPin;
  pd: TPin_Direction;
Begin
  Result := False;
  If (Filter = nil) Or Failed(Filter.EnumPins(pins)) Then
    Exit;
  If pins = nil Then
    Exit;
  pins.Reset;
  While True Do
  Begin
    If (pins.Next(1, pin, nil) <> S_OK) Then
      break;
    // stopping every pin having IAMStreamControl interface
    If Succeeded(pin.QueryDirection(pd)) And (pd = PINDIR_INPUT) Then
    Begin
      Result := ControlPin(pin, start);
    End;
  End;
End;

Function TVideoCapture.ControlCaptureStream(start: Boolean): Boolean;
Var
  pin: IPin;
  flt: IBaseFilter;
Begin
  Result := False;
{
  flt:= nil;
  if start then
    Sink.QueryInterface(IID_IBaseFilter, flt)
  else
    flt:= NullRenderer;

  if flt=nil then Exit;
  if Succeeded(Builder.FindPin(Render, PINDIR_OUTPUT, TGUID(nil^), TGUID(nil^), false, 0, pin)) then begin
    if Succeeded(pin.QueryInterface(IID_IPinFlowControl, pfc)) then begin
      if Succeeded(Graph.QueryInterface(IID_IGraphConfig, Config)) then begin
        pfc.Block(AM_PIN_FLOW_CONTROL_BLOCK, 0); // Block the data flow.
        // Reconfigure the graph.
        Result:= Succeeded( Config.Reconnect( pin, nil, nil, flt, 0, 0));
        pfc.Block(0, 0);   // Unblock the data flow.
      end;
    end;
  end;
}
  If Graph = nil Then
    Exit;
  // control multiplexer
  Result := ControlFilter(Render, start);
  // have to control smart tee capture pin
  // cause some compressors dont let to run stream after stopping MUX input(((((
  If Succeeded(Graph.FindFilterByName(SmartTeeFilterName, flt)) Then
  Begin
    // microsoft: capture is always output pin 0, and preview is output pin 1.
    If Succeeded(Builder.FindPin(flt, PINDIR_OUTPUT, TGUID(nil^), TGUID(nil^), False, 0, pin)) Then
    Begin
      // DX SDK: In addition, you should specify TRUE for the bSendExtra parameter
      //         on the capture pin, and specify FALSE to the multiplexer pin.
      ControlPin(pin, start, True);
    End;
  End;
End;

Function TVideoCapture.StartPreview: Boolean;
Var
  MC: IMediaControl;
  hr: HRESULT;
Begin
  // way ahead of you
  Result := Assigned(Graph) And Previewing;
  If Result Or (Not BuildGraph) Then
    Exit;
  // Result = false
  // stop capture streams
  If WantCapture Then
    ControlCaptureStream(False); // stop capture part of graph

  // run the graph
  hr := Graph.QueryInterface(IID_IMediaControl, MC);
  If Succeeded(hr) Then
  Begin
    hr := MC.Run;
    If Failed(hr) Then
      MC.Stop;  // stop parts that ran
  End;
  If Failed(hr) Then
  Begin
    ErrMsg(rsCantRunPreviewGraph, hr);
    Exit;
  End;
  FPreviewing := True;
  Result := True;
  If Assigned(FOnStartPreview) Then
    FOnStartPreview(Self);
End;

Function TVideoCapture.StopPreview: Boolean;
Var
  MC: IMediaControl;
  hr: HRESULT;
Begin
  Result := False; // way ahead of you
  If Not Previewing Then
    Exit;
  // stop the graph
  If Graph <> nil Then
  Begin
    hr := Graph.QueryInterface(IID_IMediaControl, MC);
    If Succeeded(hr) Then
      hr := MC.Stop;
    If Failed(hr) Then
    Begin
      ErrMsg(rsCantStopPreviewGraph, hr);
      Exit;
    End;
  End;
  Invalidate; // !!! get rid of garbage
  If Assigned(FOnStopPreview) Then
    FOnStopPreview(Self);
  FPreviewing := False;
  Result := True;
End;

// capture AVI
Function TVideoCapture.StartCapture(Const Dialog: Boolean): Boolean;
Var
  FHasStreamControl: Boolean;
  hr: HRESULT;
  MC: IMediaControl;
Begin
  // way ahead of you
  Result := False;
  If Not WantCapture Then
    Exit;

  Result := Capturing;
  If Result Then
    Exit;
  FCaptureTimer.Enabled := False;

  // or we'll crash
  Result := False;
  If Not BuildGraph Then
    Exit;

  // This amount will be subtracted from the number of dropped and not
  // dropped frames reported by the filter.  Since we might be having the
  // filter running while the pin is turned off, we don't want any of the
  // frame statistics from the time the pin is off interfering with the
  // statistics we gather while the pin is on
  FDroppedFrames := 0;
  FNotDropped := 0;

  FHasStreamControl := ControlCaptureStream(False);  // stop capture part of graph

  // prepare to run the graph
  MC := nil;
  hr := Graph.QueryInterface(IID_IMediaControl, MC);
  If (Failed(hr)) Then
  Begin
    ErrMsg(rsCantGetMediaControl, hr);
    Result := False;
    Exit;
  End;

  Try
    // If we were able to keep capture off, then we can
    // run the graph now for frame accurate start later yet still showing a
    // preview. Otherwise, we can't run the graph yet without capture
    // starting too, so we'll pause it so the latency between when they
    // press a key and when capture begins is still small (but they won't have
    // a preview while they wait to press a key)
    If (FHasStreamControl) Then
      hr := MC.Run()
    Else
      hr := MC.Pause();

    If (Failed(hr)) Then // stop parts that started
      ErrMsgException(rsCantRunGraph, hr);

    FPreviewing := False;
    // press a key to start capture
    If (Dialog And (MessageDlg(rsStartCapture, mtConfirmation, [mbYes, mbCancel], 0) <> mrYes)) Then
    Begin
      MC.Stop(); // !!! tnx to Gabriel Corneanu
      If WantPreview Then
        StartPreview;
    End
    Else
    Begin
      // Start capture NOW!
      If (FHasStreamControl) Then
      Begin
        // we may not have this yet
        If (DroppedFrames = nil) Then
        Begin
          hr := Builder.FindInterface(@PIN_CATEGORY_CAPTURE, @MEDIATYPE_Interleaved,
            VCap, IID_IAMDroppedFrames, DroppedFrames);
          If (hr <> NOERROR) Then
            Builder.FindInterface(@PIN_CATEGORY_CAPTURE, @MEDIATYPE_Video,
              VCap, IID_IAMDroppedFrames, DroppedFrames);
        End;
        // turn the capture pins on now!
        ControlCaptureStream(True);  // run capture part of graph
        // make note of the current dropped frame counts
        If (DroppedFrames <> nil) Then
        Begin
          DroppedFrames.GetNumDropped(FDroppedBase);
          DroppedFrames.GetNumNotDropped(FNotDroppedBase);
        End;
      End
      Else
      Begin
        hr := MC.Run();
        If (Failed(hr)) Then // stop parts that started
          ErrMsgException(rsCantRunGraph, hr);
      End;

  // when did we start capture?
      FCapStartTime := timeGetTime();
      FCaptureTimer.Enabled := True;
      FCapturing := True;
      Result := True;
      Try
        If Assigned(FOnStartCapture) Then
          FOnStartCapture(Self);
      Except
      End;
    End;
  Except
    MC.Stop();
    Result := False;
  End;
End;

Function TVideoCapture.StopCapture(RenameFile: String = ''): Boolean;
Var
  MC: IMediaControl;
  hr: HRESULT;
Begin
  // way ahead of you
  Result := False;
  If (Not Capturing) Then
    Exit;

  FCaptureTimer.Enabled := False;

  If Graph = nil Then
    Exit;
  // stop the graph
  hr := Graph.QueryInterface(IID_IMediaControl, MC);
  If (Succeeded(hr)) Then
  Begin
    hr := MC.Stop();
    MC := nil;
  End;
  If (Failed(hr)) Then
  Begin
    ErrMsg(rsCantStopGraph, hr);
    Exit;
  End;

  // when the graph was stopped
  FCapStopTime := timeGetTime();
  FCapturing  := False;
  FPreviewing := False;

//  if UseTempFile and SaveCaptureFile(CaptureFileName) then
//    DeleteFile(TempCaptureFileName);
//  renamefile(TempCaptureFileName, RenameFile);
  If RenameFile <> '' Then
    MoveFileEx(PChar(TempCaptureFileName), PChar(RenameFile), MOVEFILE_COPY_ALLOWED);

  // one last time for the final count and all the stats
  Try
    If Assigned(FOnCaptureProgress) Then
      FOnCaptureProgress(Self);
  Except
  End;
  // !!! get rid of garbage
  Invalidate;

  If Assigned(FOnStopCapture) Then
    FOnStopCapture(Self);

  If WantPreview Then
    StartPreview;

  Result := True;
End;

//--------------------------------------------------------------------

Procedure TVideoCapture.CaptureProgress(Sender: TObject);
Begin
  FCapTime := timeGetTime() - FCapStartTime;
  UpdateStatus;
  If Capturing Then
    If (UseTimeLimit) And (FCapTime Div 1000 >= DWORD(TimeLimit)) Then
    Begin
      // time is up!!!
      StopCapture;
      If WantPreview Then
        StartPreview;
    End
    Else
    If Assigned(FOnCaptureProgress) Then
      FOnCaptureProgress(Self);
End;

Procedure TVideoCapture.UpdateStatus;
Begin
  // this filter can't tell us dropped frame info.
  If (DroppedFrames <> nil) And FCapturing Then
  Begin
    If (DroppedFrames.GetNumDropped(FDroppedFrames) = S_OK) And
      (DroppedFrames.GetNumNotDropped(FNotDropped) = S_OK) Then
    Begin
      FDroppedFrames := FDroppedFrames - FDroppedBase;
      FNotDropped := FNotDropped - FNotDroppedBase;
    End;
{
    // we want all possible stats, including capture time and actual acheived
    // frame rate and data rate (as opposed to what we tried to get).  These
    // numbers are an indication that though we dropped frames just now, if we
    // chose a data rate and frame rate equal to the numbers I'm about to
    // print, we probably wouldn't drop any frames.
    // average size of frame captured
    if (DroppedFrames.GetAverageFrameSize(&lAvgFrameSize) = S_OK) then begin
    end;
}
  End;
End;

Procedure TVideoCapture.SetFrameRate(Const Value: Double);
Const
  DefaultFrameRate = 25;
Begin
  If (Value < 1) Or (Value > 50) Then
    FFrameRate := DefaultFrameRate
  Else
    FFrameRate := Value;
End;

Procedure TVideoCapture.SetTempCaptureFileName(Const Value: String);
Begin
  FTempCaptureFileName := Value;
  If (Sink <> nil) Then
  Begin
    Sink.SetFileName(PWCHAR(WideString(FTempCaptureFileName)), nil);
  End;
End;

Function TVideoCapture.AllocCaptureFile(Const SizeMb: Integer): Boolean;
Begin
  Result := MakeBuilder And (Builder.AllocCapFile(PWCHAR(FCaptureFile),
    SizeMb * 1024 * 1024) = NOERROR);
End;

//Function TVideoCapture.SaveCaptureFile(Const FileName: String): Boolean;
//Var
//  tempBuilder: ICaptureGraphBuilder2;
//Begin
//  Result := FCaptureFile <> '';
//  If Result Then
//    Try
//      // we need our own graph builder because the main one might not exist
//      Result := (CoCreateInstance(CLSID_CaptureGraphBuilder2, nil, CLSCTX_INPROC,
//        IID_ICaptureGraphBuilder2, tempBuilder) = NOERROR);
//      If (Result) Then
//        Try
//          // allow the user to press ESC to abort... don't ask for progress
//          Result := tempBuilder.CopyCaptureFile(PWCHAR(FCaptureFile),
//            PWCHAR(WideString(FileName)), -1, nil) = S_OK;
//        Finally
//          tempBuilder := nil;
//        End;
//    Except
//      Result := False;
//    End;
//End;

Procedure TVideoCapture.SetCaptureFileName(Const Value: String);
Begin
  FCaptureFileName := Value;
End;

Procedure TVideoCapture.GetAvailableDialogs;
Type
  chars = Array[0..32767] Of Char;
Var
  Spec: ISpecifyPropertyPages;
  StreamConf: IAMStreamConfig;
  auuid: CAUUID;
  hr: HRESULT;
  TVTuner: IAMTVTuner;
  TVAudio: IAMTVAudio;
  i:  Integer;
  p:  ^chars;
Begin
  FDialogs := [];
  If CaptureDialogs <> nil Then
  Begin
    // If this device supports the old legacy UI dialogs, offer them
    If CaptureDialogs.HasDialog(VfwCaptureDialog_Format) = S_OK Then
    Begin
      FDialogs := FDialogs + [cdVFORMAT];
    End;
    If CaptureDialogs.HasDialog(VfwCaptureDialog_Source) = S_OK Then
    Begin
      FDialogs := FDialogs + [cdVSOURCE];
    End;
    If CaptureDialogs.HasDialog(VfwCaptureDialog_Display) = S_OK Then
    Begin
      FDialogs := FDialogs + [cdVDISPLAY];
    End;
  End;

  // Also check the audio capture filter at this point, since even non wdm devices
  // may support an IAMAudioInputMixer property page (we'll also get any wdm filter
  // properties here as well). We'll get any audio capture pin property pages just
  // a bit later.
  If (ACap <> nil) And (ACap.QueryInterface(IID_ISpecifyPropertyPages, Spec) = S_OK) Then
  Begin
    If (Spec.GetPages(auuid) = S_OK) And (auuid.cElems > 0) Then
    Begin
      FDialogs := FDialogs + [cdACAPTURE];
      CoTaskMemFree(auuid.pElems);
    End;
    Spec := nil;
  End;

  If (VComp <> nil) And (VComp.QueryInterface(IID_ISpecifyPropertyPages, Spec) = S_OK) Then
  Begin
    If (Spec.GetPages(auuid) = S_OK) And (auuid.cElems > 0) Then
    Begin
      FDialogs := FDialogs + [cdVCOMPRESSION];
      CoTaskMemFree(auuid.pElems);
    End;
    Spec := nil;
  End
  Else
  Begin
    If (CompressDialogs <> nil) And
      (CompressDialogs.ShowDialog(VfwCompressDialog_QueryConfig, Handle) = S_OK) Then
    Begin
      FDialogs := FDialogs + [cdVCOMPRESSION];
      i := 0;
      If Succeeded(CompressDialogs.GetState(nil^, i)) Then
      Begin
        GetMem(p, i);
        CompressDialogs.GetState(p^, i);
        FreeMem(p);
      End;
    End;
  End;

  // don't bother looking for new property pages if the old ones are supported
  // or if we don't have a capture filter
  If (VCap = nil) Or (cdVFORMAT In FDialogs) Then
    Exit;

  // New WDM devices support new UI and new interfaces.
  // Your app can use some default property
  // pages for UI if you'd like (like we do here) or if you don't like our
  // dialog boxes, feel free to make your own and programmatically set
  // the capture options through interfaces like IAMCrossbar, IAMCameraControl
  // etc.
  // There are 9 objects that might support property pages.  Let's go through
  // them.
  // 1. the video capture filter itself

  If (VCap.QueryInterface(IID_ISpecifyPropertyPages, Spec) = S_OK) Then
  Begin
    If (Spec.GetPages(auuid) = S_OK) And (auuid.cElems > 0) Then
    Begin
      FDialogs := FDialogs + [cdVCAPTURE];
      CoTaskMemFree(auuid.pElems);
    End;
    Spec := nil;
  End;

  // 2.  The video capture capture pin
  hr := Builder.FindInterface(@PIN_CATEGORY_CAPTURE, @MEDIATYPE_Interleaved, VCap, IID_IAMStreamConfig, StreamConf);
  If hr <> S_OK Then
    hr := Builder.FindInterface(@PIN_CATEGORY_CAPTURE, @MEDIATYPE_Video, VCap, IID_IAMStreamConfig, StreamConf);
  If hr = S_OK Then
  Begin
    If StreamConf.QueryInterface(IID_ISpecifyPropertyPages, Spec) = S_OK Then
    Begin
      If (Spec.GetPages(auuid) = S_OK) And (auuid.cElems > 0) Then
      Begin
        FDialogs := FDialogs + [cdVCAPTURE_PIN];
        CoTaskMemFree(auuid.pElems);
      End;
      Spec := nil;
    End;
    StreamConf := nil;
  End;

  // 3.  The video capture preview pin.

  // This basically sets the format being previewed.  Typically, you
  // want to capture and preview using the SAME format, instead of having to
  // enter the same value in 2 dialog boxes.  For a discussion on this, see
  // the comment above the MakePreviewGraph function.
  hr := Builder.FindInterface(@PIN_CATEGORY_PREVIEW, @MEDIATYPE_Interleaved, VCap, IID_IAMStreamConfig, StreamConf);
  If hr <> S_OK Then
    hr := Builder.FindInterface(@PIN_CATEGORY_PREVIEW, @MEDIATYPE_Video, VCap, IID_IAMStreamConfig, StreamConf);
  If hr = S_OK Then
  Begin
    If StreamConf.QueryInterface(IID_ISpecifyPropertyPages, Spec) = S_OK Then
    Begin
      If (Spec.GetPages(auuid) = S_OK) And (auuid.cElems > 0) Then
      Begin
        FDialogs := FDialogs + [cdVPREVIEW_PIN];
        CoTaskMemFree(auuid.pElems);
      End;
      Spec := nil;
    End;
    StreamConf := nil;
  End;

  // 4 & 5.  The video crossbar, and a possible second crossbar

  // 6.  The TVTuner
  hr := Builder.FindInterface(@PIN_CATEGORY_CAPTURE, @MEDIATYPE_Interleaved, VCap, IID_IAMTVTuner, TVTuner);
  If hr <> S_OK Then
    hr := Builder.FindInterface(@PIN_CATEGORY_CAPTURE, @MEDIATYPE_Video, VCap, IID_IAMTVTuner, TVTuner);
  If hr = S_OK Then
  Begin
    If TVTuner.QueryInterface(IID_ISpecifyPropertyPages, Spec) = S_OK Then
    Begin
      If (Spec.GetPages(auuid) = S_OK) And (auuid.cElems > 0) Then
      Begin
        FDialogs := FDialogs + [cdTVTUNER];
        CoTaskMemFree(auuid.pElems);
      End;
      Spec := nil;
    End;
    TVTuner := nil;
  End;

  // no audio capture, we're done
  If ACap = nil Then
    Exit;

  If (AComp <> nil) And (AComp.QueryInterface(IID_ISpecifyPropertyPages, Spec) = S_OK) Then
  Begin
    If (Spec.GetPages(auuid) = S_OK) And (auuid.cElems > 0) Then
    Begin
      FDialogs := FDialogs + [cdACOMPRESSION];
      CoTaskMemFree(auuid.pElems);
    End;
    Spec := nil;
  End;

  // 7.  The Audio capture filter itself... Thanks anyway, but we got these already

  // 8.  The Audio capture pin
  If Builder.FindInterface(@PIN_CATEGORY_CAPTURE, @MEDIATYPE_Audio, ACap, IID_IAMStreamConfig, StreamConf) = S_OK Then
  Begin
    If StreamConf.QueryInterface(IID_ISpecifyPropertyPages, Spec) = S_OK Then
    Begin
      If (Spec.GetPages(auuid) = S_OK) And (auuid.cElems > 0) Then
      Begin
        FDialogs := FDialogs + [cdACAPTURE_PIN];
        CoTaskMemFree(auuid.pElems);
      End;
      Spec := nil;
    End;
    StreamConf := nil;
  End;

  // 9.  The TV Audio filter
  If Builder.FindInterface(@PIN_CATEGORY_CAPTURE, @MEDIATYPE_Audio, ACap, IID_IAMTVAudio, TVAudio) = S_OK Then
  Begin
    If TVAudio.QueryInterface(IID_ISpecifyPropertyPages, Spec) = S_OK Then
    Begin
      If (Spec.GetPages(auuid) = S_OK) And (auuid.cElems > 0) Then
      Begin
        FDialogs := FDialogs + [cdTVAUDIO];
        CoTaskMemFree(auuid.pElems);
      End;
      Spec := nil;
    End;
    TVAudio := nil;
  End;
End;

Function TVideoCapture.ShowDialog(Const DialogType: TCaptureDialog): Boolean;
Var
  Spec: ISpecifyPropertyPages;
  auuid: CAUUID;
  hr: HRESULT;
  StreamConf: IAMStreamConfig;
  TVTuner: IAMTVTuner;
Begin
  Result := False;
  If Not (DialogType In FDialogs) Or FCapturing Then
    Exit;

  // they want the VfW format dialog
  Case DialogType Of
    cdVFORMAT:
    Begin
        // this dialog will not work while previewing
        // Oh uh!  Sometimes bringing up the FORMAT dialog can result
        // in changing to a capture format that the current graph
        // can't handle.  It looks like that has happened and we'll
        // have to rebuild the graph.
      If CaptureDialogs.ShowDialog(VfwCaptureDialog_Format, Handle) = VFW_E_CANNOT_CONNECT Then
        TearDownGraph;    // now we need to rebuild
      If WantPreview Then
        StartPreview;
    End;
    cdVSOURCE:
    Begin
        // this dialog will not work while previewing
      StopPreview;
      CaptureDialogs.ShowDialog(VfwCaptureDialog_Source, Handle);
      If WantPreview Then
        StartPreview;
    End;
    cdVDISPLAY:
    Begin
        // this dialog will not work while previewing
      StopPreview;
      CaptureDialogs.ShowDialog(VfwCaptureDialog_Display, Handle);
      If WantPreview Then
        StartPreview;
    End;
    // now the code for the new dialogs
    cdVCAPTURE:
    Begin
      If VCap.QueryInterface(IID_ISpecifyPropertyPages, Spec) = S_OK Then
      Begin
        Spec.GetPages(auuid);
        OleCreatePropertyFrame(Handle, 30, 30, nil, 1, @VCap, auuid.cElems, auuid.pElems, 0, 0, nil);
        CoTaskMemFree(auuid.pElems);
        Spec := nil;
      End;
    End;
    cdVCAPTURE_PIN:
    Begin
        // You can change this pin's output format in these dialogs.
        // If the capture pin is already connected to somebody who's
        // fussy about the connection type, that may prevent using
        // this dialog(!) because the filter it's connected to might not
        // allow reconnecting to a new format. (EG: you switch from RGB
        // to some compressed type, and need to pull in a decoder)
        // I need to tear down the graph downstream of the
        // capture filter before bringing up these dialogs.
        // In any case, the graph must be STOPPED when calling them.
        // The capture pin that we are trying to set the format on is connected if
        // one of these variable is set to TRUE. The pin should be disconnected for
        // the dialog to work properly.
      TearDownGraph;    // graph could prevent dialog working
//        hr:= Builder.FindInterface(@PIN_CATEGORY_CAPTURE, @MEDIATYPE_Interleaved, VCap, IID_IAMStreamConfig, StreamConf);
//        if hr<>S_OK then
      Builder.FindInterface(@PIN_CATEGORY_CAPTURE, @MEDIATYPE_Video, VCap, IID_IAMStreamConfig, StreamConf);
      If StreamConf.QueryInterface(IID_ISpecifyPropertyPages, Spec) = S_OK Then
      Begin
        Spec.GetPages(auuid);
        OleCreatePropertyFrame(Handle, 30, 30, nil, 1, @StreamConf, auuid.cElems, auuid.pElems, 0, 0, nil);
        CoTaskMemFree(auuid.pElems);
        Spec := nil;
      End;
      StreamConf := nil;
      If WantPreview Then
        StartPreview;
    End;
    cdVPREVIEW_PIN:
    Begin
        // this dialog may not work if the preview pin is connected
        // already, because the downstream filter may reject a format
        // change, so we better kill the graph. (EG: We switch from
        // capturing RGB to some compressed fmt, and need to pull in
        // a decompressor)
      TearDownGraph;
        // This dialog changes the preview format, so it might affect
        // the format being drawn.  Our app's window size is taken
        // from the size of the capture pin's video, not the preview
        // pin, so changing that here won't have any effect. All in all,
        // this probably won't be a terribly useful dialog in this app.
//        hr:= Builder.FindInterface(@PIN_CATEGORY_PREVIEW, @MEDIATYPE_Interleaved, VCap, IID_IAMStreamConfig, StreamConf);
//        if hr<>S_OK then
      Builder.FindInterface(@PIN_CATEGORY_PREVIEW, @MEDIATYPE_Video, VCap, IID_IAMStreamConfig, StreamConf);
      If StreamConf.QueryInterface(IID_ISpecifyPropertyPages, Spec) = S_OK Then
      Begin
        Spec.GetPages(auuid);
        OleCreatePropertyFrame(Handle, 30, 30, nil, 1, @StreamConf, auuid.cElems, auuid.pElems, 0, 0, nil);
        CoTaskMemFree(auuid.pElems);
        Spec := nil;
      End;
      StreamConf := nil;
      If WantPreview Then
        StartPreview;
    End;
    cdTVTUNER:
    Begin
      hr := Builder.FindInterface(@PIN_CATEGORY_CAPTURE, @MEDIATYPE_Interleaved, VCap, IID_IAMTVTuner, TVTuner);
      If hr <> S_OK Then
        Builder.FindInterface(@PIN_CATEGORY_CAPTURE, @MEDIATYPE_Video, VCap, IID_IAMTVTuner, TVTuner);
      If TVTuner.QueryInterface(IID_ISpecifyPropertyPages, Spec) = S_OK Then
      Begin
        Spec.GetPages(auuid);
        OleCreatePropertyFrame(Handle, 30, 30, nil, 1, @TVTuner, auuid.cElems, auuid.pElems, 0, 0, nil);
        CoTaskMemFree(auuid.pElems);
        Spec := nil;
      End;
      TVTuner := nil;
    End;
    cdACAPTURE:
    Begin
      If ACap.QueryInterface(IID_ISpecifyPropertyPages, Spec) = S_OK Then
      Begin
        Spec.GetPages(auuid);
        OleCreatePropertyFrame(Handle, 30, 30, nil, 1, @ACap, auuid.cElems, auuid.pElems, 0, 0, nil);
        CoTaskMemFree(auuid.pElems);
        Spec := nil;
      End;
    End;
    cdACAPTURE_PIN:
    Begin
        // this dialog will not work while previewing - it might change
        // the output format!
      StopPreview;
      Builder.FindInterface(@PIN_CATEGORY_CAPTURE, @MEDIATYPE_Audio, ACap, IID_IAMStreamConfig, StreamConf);
      If StreamConf.QueryInterface(IID_ISpecifyPropertyPages, Spec) = S_OK Then
      Begin
        Spec.GetPages(auuid);
        OleCreatePropertyFrame(Handle, 30, 30, nil, 1, @StreamConf, auuid.cElems, auuid.pElems, 0, 0, nil);
        CoTaskMemFree(auuid.pElems);
        Spec := nil;
      End;
      StreamConf := nil;
      If WantPreview Then
        StartPreview;
    End;
    cdVCOMPRESSION:
    Begin
      StopPreview;
      If (CompressDialogs <> nil) Then
        CompressDialogs.ShowDialog(VfwCompressDialog_Config, Handle)
      Else
      If VComp.QueryInterface(IID_ISpecifyPropertyPages, Spec) = S_OK Then
      Begin
        Spec.GetPages(auuid);
        OleCreatePropertyFrame(Handle, 30, 30, nil, 1, @VComp, auuid.cElems, auuid.pElems, 0, 0, nil);
        CoTaskMemFree(auuid.pElems);
        Spec := nil;
      End;
      If WantPreview Then
        StartPreview;
    End;
    cdACOMPRESSION:
    Begin
      StopPreview;
      If AComp.QueryInterface(IID_ISpecifyPropertyPages, Spec) = S_OK Then
      Begin
        Spec.GetPages(auuid);
        OleCreatePropertyFrame(Handle, 30, 30, nil, 1, @AComp, auuid.cElems, auuid.pElems, 0, 0, nil);
        CoTaskMemFree(auuid.pElems);
        Spec := nil;
      End;
      If WantPreview Then
        StartPreview;
    End;
  End;
  Result := True;
End;

// saving/restoring graph functions

Procedure TVideoCapture.RestoreGraph(config: TGraphConfig);
Begin
  If (config = nil) Then
    Exit;
  TearDownGraph;
  FreeCapFilters;

  UseTempFile := config.UseTempFile;
  WantBitmaps := config.WantBitmaps;
  DoPreallocFile := config.DoPreallocFile;
  WantCapture := config.WantCapture;
  WantPreview := config.WantPreview;
  WantDVAudio := config.WantDVAudio;
  WantAudio := config.WantAudio;
  WantAudioPreview := config.WantAudioPreview;
  PreallocFileSize := config.PreallocFileSize;
  DVResolution := config.DVResolution;
  BitmapPixelFormat := config.PixelFormat;
  If config.TempCaptureFileName <> '' Then
    TempCaptureFileName := config.TempCaptureFileName;
  If config.CaptureFileName <> '' Then
    CaptureFileName := config.CaptureFileName;

  BuildDevicesList;
  mVideo := GetMonikerByString(VideoDevicesList, config.VCapSource);
  mAudio := GetMonikerByString(AudioDevicesList, config.ACapSource);
  mAComp := GetMonikerByString(AudioCompressorsList, config.AComp);
  mVComp := GetMonikerByString(VideoCompressorsList, config.VComp);

  Asm //this was added due to corrupted registers after func call (D5 compiler error?)
    Push Ebx;
    Push Edi;
    Push Esi
  End;
  InitCapFilters;
  Asm
    Pop Esi;
    Pop Edi;
    Pop Ebx
  End;

  // Get VComp State
  If (config.VCompState <> '') And (CompressDialogs <> nil) Then
  Begin
    CompressDialogs.SetState(PChar(config.VCompState)^, Length(config.VCompState));
  End;

  SetACapMode(config.ACapMode);
  SetVCapMode(config.VCapMode);

  If BuildGraph And WantPreview Then
    StartPreview;

  If Assigned(FOnChangeDevice) Then
    FOnChangeDevice(Self);
  If Assigned(FOnChangeCompressor) Then
    FOnChangeCompressor(Self);
End;

Procedure TVideoCapture.SaveGraph(config: TGraphConfig);
Var
  i: Integer;
Begin
  If (config = nil) Then
    Exit;
  config.Clear;
  config.UseTempFile := UseTempFile;
  config.WantBitmaps := WantBitmaps;
  config.DoPreallocFile := DoPreallocFile;
  config.WantCapture := WantCapture;
  config.WantPreview := WantPreview;
  config.WantDVAudio := WantDVAudio;
  config.WantAudio := WantAudio;
  config.WantAudioPreview := WantAudioPreview;
  config.PreallocFileSize := PreallocFileSize;
  config.TempCaptureFileName := TempCaptureFileName;
  config.CaptureFileName := CaptureFileName;
  config.DVResolution := DVResolution;
  config.PixelFormat := BitmapPixelFormat;
  config.ACapMode  := ACapMode;
  config.VCapMode  := VCapMode;
  config.VCompState := '';

  // Set VComp State
  If (CompressDialogs <> nil) Then
  Begin
    If Succeeded(CompressDialogs.GetState(nil^, i)) Then
    Begin
      SetLength(config.VCompState, i);
      CompressDialogs.GetState(PChar(config.VCompState)^, i);
    End;
  End;

  config.VCapSource := VCapName;
  config.ACapSource := ACapName;
  config.VComp := VCompName;
  config.AComp := ACompName;
End;

//------------------------------------------------------------------------------
// TGraphConfig - helper class to save/restore TVideoCapture settings

Constructor TGraphConfig.Create;
Begin
  Clear;
End;

Procedure TGraphConfig.Clear;
Begin
  WantBitmaps := False;
  WantCapture := False;
  WantPreview := False;
  WantDVAudio := False;
  WantAudio := False;
  WantAudioPreview := True;
  DoPreallocFile := False;
  PreallocFileSize := 0;
  UseTempFile := False;
  TempCaptureFileName := 'captemp.avi';
  CaptureFileName := 'capture.avi';
  ZeroMemory(@ACapMode, sizeof(ACapMode));
  ZeroMemory(@VCapMode, sizeof(VCapMode));
  DVResolution := dvrFull;
  PixelFormat := pf24bit;
  VCapSource := '';
  ACapSource := '';
  AComp := '';
  VComp := '';
End;

Const
  cWantBitmaps = 'WantBitmaps';
  cWantCapture = 'WantCapture';
  cWantPreview = 'WantPreview';
  cWantDVAudio = 'WantDVAudio';
  cWantAudio = 'WantAudio';
  cWantAudioPreview = 'WantAudioPreview';
  cDoPreallocFile = 'DoPreallocFile';
  cPreallocFileSize = 'PreallocFileSize';
  cUseTempFile = 'UseTempFile';
  cTempCaptureFileName = 'TempCaptureFileName';
  cCaptureFileName = 'CaptureFileName';
  cACapModeMediaSubType = 'ACapModeMediaSubType';
  cACapModeSampleRate = 'ACapModeSampleRate';
  cACapModeAvgSampleRate = 'ACapModeAvgSampleRate';
  cACapModeSampleSize = 'ACapModeSampleSize';
  cACapModeChannels = 'ACapModeChannels';
  cVCapModeMediaSubType = 'VCapModeMediaSubType';
  cVCapModeWidth = 'VCapModeWidth';
  cVCapModeHeight = 'VCapModeHeight';
  cVCapModeBits = 'VCapModeBits';
  cDVResolution = 'DVResolution';
  cPixelFormat = 'PixelFormat';
  cVCapSource = 'VCapSource';
  cACapSource = 'ACapSource';
  cAComp = 'AComp';
  cVComp = 'VComp';
  cVCompState = 'VCompState';

Procedure TGraphConfig.RestoreGraph(layout: String);
Var
  sl: TStringList;
Begin
  Clear;
  sl := TStringList.Create;
  Try
    sl.CommaText := layout;
    WantBitmaps := StrToIntDef(sl.Values[cWantBitmaps], 1) > 0;
    WantCapture := StrToIntDef(sl.Values[cWantCapture], 1) > 0;
    WantPreview := StrToIntDef(sl.Values[cWantPreview], 1) > 0;
    WantAudio := StrToIntDef(sl.Values[cWantAudio], 1) > 0;
    WantAudioPreview := StrToIntDef(sl.Values[cWantAudioPreview], 1) > 0;
    WantDVAudio := StrToIntDef(sl.Values[cWantDVAudio], 1) > 0;
    DoPreallocFile := StrToIntDef(sl.Values[cDoPreallocFile], 1) > 0;
    UseTempFile := StrToIntDef(sl.Values[cUseTempFile], 1) > 0;
    PreallocFileSize := StrToIntDef(sl.Values[cPreallocFileSize], 0);
    TempCaptureFileName := sl.Values[cTempCaptureFileName];
    If TempCaptureFileName = '' Then
      TempCaptureFileName := 'captemp.avi';
    CaptureFileName := sl.Values[cCaptureFileName];
    If CaptureFileName = '' Then
      CaptureFileName := 'capture.avi';
    DVResolution := TDVResolution(StrToIntDef(sl.Values[cDVResolution], Integer(dvrFull)));
    If Not (DVResolution In [low(TDVResolution)..high(TDVResolution)]) Then
      DVResolution := dvrFull;
    PixelFormat := TPixelFormat(StrToIntDef(sl.Values[cPixelFormat], Integer(pf24bit)));
    If Not (PixelFormat In [low(TPixelFormat)..high(TPixelFormat)]) Then
      PixelFormat := pf24bit;
    VCapSource := sl.Values[cVCapSource];
    ACapSource := sl.Values[cACapSource];
    AComp := sl.Values[cAComp];
    VComp := sl.Values[cVComp];
    ACapMode.MediaSubType := StrToGUID(sl.Values[cACapModeMediaSubType]);
    ACapMode.SampleRate := StrToIntDef(sl.Values[cACapModeSampleRate], 0);
    ACapMode.AvgSampleRate := StrToIntDef(sl.Values[cACapModeAvgSampleRate], 0);
    ACapMode.SampleSize := StrToIntDef(sl.Values[cACapModeSampleSize], 0);
    ACapMode.Channels := StrToIntDef(sl.Values[cACapModeChannels], 0);
    VCapMode.MediaSubType := StrToGUID(sl.Values[cVCapModeMediaSubType]);
    VCapMode.Width := StrToIntDef(sl.Values[cVCapModeWidth], 0);
    VCapMode.Height := StrToIntDef(sl.Values[cVCapModeHeight], 0);
    VCapMode.BitCount := StrToIntDef(sl.Values[cVCapModeBits], 0);
    VCompState := StrToBytes(sl.Values[cVCompState]);
  Finally
    sl.Free;
  End;
End;

Function TGraphConfig.SaveGraph: String;
Var
  sl: TStringList;
Begin
  sl := TStringList.Create;
  Try
    If Not WantBitmaps Then
      sl.Add(cWantBitmaps + '=0');
    If Not WantCapture Then
      sl.Add(cWantCapture + '=0');
    If Not WantPreview Then
      sl.Add(cWantPreview + '=0');
    If Not WantDVAudio Then
      sl.Add(cWantDVAudio + '=0');
    If Not WantAudio Then
      sl.Add(cWantAudio + '=0');
    If Not WantAudioPreview Then
      sl.Add(cWantAudioPreview + '=0');
    If Not DoPreallocFile Then
      sl.Add(cDoPreallocFile + '=0');
    If Not UseTempFile Then
      sl.Add(cUseTempFile + '=0');
    sl.Add(cPreallocFileSize + '=' + IntToStr(PreallocFileSize));
    sl.Add(cTempCaptureFileName + '=' + TempCaptureFileName);
    sl.Add(cCaptureFileName + '=' + CaptureFileName);
    sl.Add(cDVResolution + '=' + IntToStr(Integer(DVResolution)));
    sl.Add(cPixelFormat + '=' + IntToStr(Integer(PixelFormat)));
    sl.Add(cVCapSource + '=' + VCapSource);
    sl.Add(cACapSource + '=' + ACapSource);
    sl.Add(cAComp + '=' + AComp);
    sl.Add(cVComp + '=' + VComp);
    sl.Add(cACapModeMediaSubType + '=' + GUIDToStr(ACapMode.MediaSubType));
    sl.Add(cACapModeSampleRate + '=' + IntToStr(ACapMode.SampleRate));
    sl.Add(cACapModeAvgSampleRate + '=' + IntToStr(ACapMode.AvgSampleRate));
    sl.Add(cACapModeSampleSize + '=' + IntToStr(ACapMode.SampleSize));
    sl.Add(cACapModeChannels + '=' + IntToStr(ACapMode.Channels));
    sl.Add(cVCapModeMediaSubType + '=' + GUIDToStr(VCapMode.MediaSubType));
    sl.Add(cVCapModeWidth + '=' + IntToStr(VCapMode.Width));
    sl.Add(cVCapModeHeight + '=' + IntToStr(VCapMode.Height));
    sl.Add(cVCapModeBits + '=' + IntToStr(VCapMode.BitCount));
    sl.Add(cVCompState + '=' + BytesToStr(VCompState));
    Result := sl.CommaText;
  Finally
    sl.Free;
  End;
End;

Function TGraphConfig.RestoreGraphFromStream(Stream: TStream): Boolean;
Var
  i: Integer;
  s: String;
Begin
  Result := False;
  If Not Assigned(Stream) Then
    Exit;
  Result := (Stream.Read(i, sizeof(i)) = sizeof(i));
  If Result Then
  Begin
    SetLength(s, i);
    Result := (Stream.Read(PChar(s)^, i) = i);
    RestoreGraph(s);
  End;
End;

Function TGraphConfig.SaveGraphToStream(Stream: TStream): Boolean;
Var
  i: Integer;
  s: String;
Begin
  Result := False;
  If Not Assigned(Stream) Then
    Exit;
  s := SaveGraph;
  i := Length(s);
  Result := (Stream.Write(i, sizeof(i)) = sizeof(i));
  Result := Result And (Stream.Write(PChar(s)^, i) = i);
End;

Initialization
  VideoDevicesList := TObjectList.Create;
  AudioDevicesList := TObjectList.Create;
  VideoCompressorsList := TObjectList.Create;
  AudioCompressorsList := TObjectList.Create;

Finalization
  VideoDevicesList.Free;
  AudioDevicesList.Free;
  VideoCompressorsList.Free;
  AudioCompressorsList.Free;
End.

